function BulkMailer(gtdinboxPage){
	this._queue = [];
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(this);
	this._sending = {};
	
	this._time = (10 * 1000);
	this._break = (2*60*1000);
	
	this._gtdinboxPage.addViewChangeCallback("all", GIPROTOTYPE.bind(this, this.viewChange));
};

BulkMailer.prototype.show = function(){

try{
	var el = this._doc.createElement("DIV");
		el.className = "bulk-mailer-send";
	
	var elTitle = this._doc.createElement("DIV");
		elTitle.innerHTML = "Subject:";
	this.elSubject = this._doc.createElement("INPUT");
		this.elSubject.type = "TEXT";
		this.elSubject.className = "subject";
	el.appendChild( elTitle );
	el.appendChild( this.elSubject );
	
	var elTitle = this._doc.createElement("DIV");
		elTitle.innerHTML = "Message:";
	this.elBody = this._doc.createElement("TEXTAREA");
		this.elBody.className = "body";
	el.appendChild( elTitle );
	el.appendChild( this.elBody );
	
	var elTitle = this._doc.createElement("DIV");
		elTitle.innerHTML = "Recipients:";
	this.elRecipients = this._doc.createElement("TEXTAREA");
		this.elRecipients.className = "recipients";
	el.appendChild( elTitle );
	el.appendChild( this.elRecipients );
	
	this.elSend = this._doc.createElement("BUTTON");
		this.elSend.innerHTML = "Send";
	el.appendChild( this.elSend );
	this.elCancel = this._doc.createElement("BUTTON");
		this.elCancel.innerHTML = "Cancel";
	el.appendChild( this.elCancel );
	this.elClose = this._doc.createElement("BUTTON");
		this.elClose.innerHTML = "Close";
	el.appendChild( this.elClose );
	
	this._eventObservers.observe( this.elSend, "mousedown", this._clickSend, false);
	this._eventObservers.observe( this.elCancel, "mousedown", GIPROTOTYPE.bind(this, function(){
			try{
				for( s in this._sending ){
					this._sending[s].recipients.length = 0;
					this._sending[s].tidyUp();
				}
				this._sending = {};			
			}catch(e){
				if( GTDDEBUGMODE ) alert("BulkMailer show cancel inner\n" + e.toString());
			}
		}), false);
	this._eventObservers.observe( this.elClose, "mousedown", GIPROTOTYPE.bind(this, function(){
			try{
				for( s in this._sending ){
					this._sending[s].recipients.length = 0;
				}
				this._sending = {};
				this._gtdinboxPage._lightbox.active(false);
			
			}catch(e){
				if( GTDDEBUGMODE ) alert("BulkMailer show close inner\n" + e.toString());
			}
		}), false);
	
	
	this._gtdinboxPage._lightbox.setType( el );
	this._gtdinboxPage._lightbox.active(true);
	
}catch(e){
	if( GTDDEBUGMODE ) alert("BulkMailer show\n" + e.toString());
}
};

BulkMailer.prototype._clickSend = function(event){

try{
	if( !this.elSubject.value || !this.elBody.value || !this.elRecipients.value ){
		alert("Please enter recipients, subject and a message.");
		return;
	}
	if( !confirm("Send this email to these recipients?") ) return;
	
	var id = (new Date()).valueOf();
	
	this.elSubject.disabled = true;
	this.elBody.disabled = true;
	this.elRecipients.disabled = true;
	this.elSend.disabled = true;
	this.elSend.innerHTML = "Sending...";
	
	this._sending[id] = {'recipients': this.elRecipients.value.split("\n"), 'subject':this.elSubject.value, 'body':this.elBody.value}
	this._sending[id].nextRecipient = GIPROTOTYPE.bind(this, function(){
		if( Math.random()>0.925 ){ // take a break
			this.elSend.innerHTML = "Holding...";
			setTimeout( this._sending[id].nextRecipient, this._break );
			return;
		}
		var neg = Math.random()>0.5? 1 : -1;
		var time = Math.floor(this._time + ((this._time * 0.25) * Math.random() * neg));
		if( this._sending[id].recipients.length > 0 ){
			setTimeout( this._sending[id].send, time );
		}else{
			this._sending[id].tidyUp();
		}
	});
	this._sending[id].send = GIPROTOTYPE.bind( this, function(){
			try{
				var s = this._sending[id];
				
				if( s.recipients.length>0 ){
					var recipient = s.recipients[0].split(",");
					var subject = s.subject;
					var msg = s.body;
					for( var v = 0; v < 40; v++ ){
						var re = new RegExp("\\<\\!\\%" + v + "\\>", "g");
						if( recipient[v] ){
							msg = msg.replace(re, GIPROTOTYPE.trim(recipient[v]) );
							subject = subject.replace(re, GIPROTOTYPE.trim(recipient[v]) );
						}
					}
					//var msg = recipient.length>1? s.body.replace("<!Name!>", GIPROTOTYPE.trim(recipient[1]) ) : s.body;
					this.sendMail( GIPROTOTYPE.trim(recipient[0]), subject, msg );
					s.recipients.splice(0,1);
				}
				if( s.recipients.length==0 ){
					//clearInterval( s.timer );
					delete this._sending[id];
					s.tidyUp();
				}else{
					this.elRecipients.value = s.recipients.join("\n");
					this.elSend.innerHTML = "Sending... (" + s.recipients.length + " remaining)";
					s.nextRecipient();
				}				
				
			}catch(e){
				if( GTDDEBUGMODE ) alert("BulkMailer _clickSend inner\n" + e.toString());
			}
		});
	this._sending[id].tidyUp = GIPROTOTYPE.bind( this, function(){
			this.elSend.innerHTML = "Send";
			this.elRecipients.value = "";
			this.elSubject.disabled = false;
			this.elBody.disabled = false;
			this.elRecipients.disabled = false;
			this.elSend.disabled = false;
		});
	//this._sending[id].timer = setInterval( this._sending[id].send, 5000 );
	this._sending[id].nextRecipient();

}catch(e){
	if( GTDDEBUGMODE ) alert("BulkMailer _clickSend\n" + e.toString());
}
};

BulkMailer.prototype.viewChange = function(){
try{
	if( this._gtdinboxPage.view("view")=="co" ){
		this._postponeSendMailWithCompose();
	}else{
		this._sendComplete();
	}
	
}catch(e){
	if( GTDDEBUGMODE ) alert("BulkMailer viewChange\n" + e.toString());
}
};

BulkMailer.prototype.sendMail = function(recipients, subject, body){
	if( recipients && body ){
		if( typeof recipients=="string" ) recipients = [recipients];
		var recipient = recipients.join(", ");
		
		this._queue.push( {'id':(new Date()).valueOf(), 'recipients':recipients, 'subject':subject, 'body':body} );
	}
	
		
	if( this._queue.length>0 ){
		if( this._gtdinboxPage.view("view")=="co" ){
			this._postponeSendMailWithCompose();
		}else{
			var elCompose = this._gtdinboxPage.gmailDOM.element("compose-mail");
			GIPROTOTYPE.doMouseEvent(this._doc, elCompose, "click");
		}
	}
};

BulkMailer.prototype._postponeSendMailWithCompose = function(){
	setTimeout( GIPROTOTYPE.bind( this, this._sendMailWithCompose ), 1000 );
};
BulkMailer.prototype._sendMailWithCompose = function(){
try{
	
	if( this._queue.length==0 ) return;
	if( this._email ){
		this._postponeSendMailWithCompose();
		return;
	}
	
	// Test UI is fully loadded
	var elBody = this._gtdinboxPage.gmailDOM.element("rich-textarea");
	var elPlainText = this._gtdinboxPage.gmailDOM.element("plain-textarea");
	
	var isRich = true;
	var elPlainTextParent = elPlainText.parentNode;
	while( elPlainTextParent.parentNode ){
		if( elPlainTextParent.style.display=="none" ){
			break;
		}
		elPlainTextParent = elPlainTextParent.parentNode;
	}
	if( elPlainTextParent.style.display!="none" ) isRich = false;
	
	if( (!elBody && isRich) || (!elPlainText && !isRich) ){
		this._postponeSendMailWithCompose();
		return;
	}
	
	// Send it
	this._email = this._queue[0];
	this._email.start = (new Date()).valueOf();
	
	var elTo = this._gtdinboxPage.gmailDOM.element("compose-to");
	var elSubject = this._gtdinboxPage.gmailDOM.element("compose-subject");
	
	elTo.value = this._email.recipients;
	elSubject.value = this._email.subject;
	
	
	if( elBody ){
		elBody.innerHTML = this._email.body.replace(/\n/g, "<br/>");
	}else{
		if( elPlainText ){
			elPlainText.value = this._email.body;
		}
	}
	
	var elSend = this._gtdinboxPage.gmailDOM.element("compose-send-first");
	GIPROTOTYPE.doMouseEvent(this._doc, elSend, "click");
	
	var id = this._email.id;
	setTimeout( GIPROTOTYPE.bind(this, function(){
			if( this._email && this._email.id==id ){	// Still here
				this._sendComplete();
			}
		}), 20000 );

}catch(e){
	if( GTDDEBUGMODE ) alert("BulkMailer sendMailWithCompose\n" + e.toString());
}
};

BulkMailer.prototype._sendComplete = function(){
try{

	if( this._email ){	// Has returned to the inbox
		for( var i = 0; i < this._queue.length; i++ ){
			if( this._queue[i].id==this._email.id ){
				this._queue.splice(i,1);
				this._email = null;
				break;
			}
		}
	}
	if( this._queue.length>0 ) this.sendMail();	// Try to load the next one
}catch(e){
	if( GTDDEBUGMODE ) alert("BulkMailer sendComplete\n" + e.toString());
}
};