/* Copyright Andy Mitchell 2007. No warranties */

function FilterPopupUI2(gtdinboxPage, name, type, elParentPopuper, parentNames, parentTypes, parentSearch, filterParent){
try{
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._labels = gtdinboxPage._labels;
	this._parentNames = parentNames || [];
	this._parentTypes = parentTypes || [type];
	this._parentSearch = parentSearch || "";
	if( name instanceof Array ){	// contact, [name,email]
		this._name = name[0];
		this._email = name[1];
		//if( !this._email ) this._email = this._sp.getEmailForContactName(this._name);
	}else{
		this._name = name;
	}
	this._type = type;
	this._elParentPopuper = elParentPopuper;
	this._filterParent = filterParent;
	if( filterParent ) this._filterParent.highlighted(false);
	
	
	this._eventObservers = new GtdEventObservers(this);
	this._showHiders = [];
	
	
	this._status = {};	// Cache for tidier code
	this._status.prefix = this._gtdinboxPage.settings.labels.types.Statuses.prefix;
	this._status.NextAction = this._status.prefix + this._gtdinboxPage.settings.labels.types.Statuses.defaults.NextAction;
	this._status.Action = this._status.prefix + this._gtdinboxPage.settings.labels.types.Statuses.defaults.Action;
	this._status.WaitingOn = this._status.prefix + this._gtdinboxPage.settings.labels.types.Statuses.defaults.WaitingOn;
	this._status.SomeDay = this._status.prefix + this._gtdinboxPage.settings.labels.types.Statuses.defaults.SomeDay;
	this._status.Finished = this._status.prefix + this._gtdinboxPage.settings.labels.types.Statuses.defaults.Finished;
	this._status.orQuery = "((" + GIPROTOTYPE.searchableLabel(this._status.NextAction) + " OR " + GIPROTOTYPE.searchableLabel(this._status.Action) + " OR " + GIPROTOTYPE.searchableLabel(this._status.WaitingOn) + " OR " + GIPROTOTYPE.searchableLabel(this._status.SomeDay) + ") -" + GIPROTOTYPE.searchableLabel(this._status.Finished) + ")";
	
	this._create();
	this._eventObservers.observe( this.el, "click", this.mouseDown, true);
	this._eventObservers.observe( this.el, "contextmenu", this.contextMenu, true);
	
	this.highlighted(true);
	this.addPopupChildObject( this );
	
	return this;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 construct\n" + e.toString());
}
};

FilterPopupUI2.prototype._create = function(){
try{
	this.el = this._doc.createElement("DIV");
		this.el.className = "filter-popup";
	var doc = this._gtdinboxPage.gmailDOM.element("document");
	if( doc ) doc.body.appendChild( this.el );
	
	var elTitle = this._doc.createElement("DIV");
		elTitle.className = "title";
		if( this._parentNames.length > 0 ){
			elTitle.innerHTML = "<div class='path'>" + this._parentNames.join(", ") + "</div>";
		}
		if( this._type=="Search" ){
			elTitle.innerHTML = "<div class='path'>" + this._parentSearch + "</div>";
		}else{
			elTitle.innerHTML += (this._name || this._email || "");
		}
	this.el.appendChild( elTitle );
	var elClose = this._doc.createElement("IMG");
		elClose.src = "chrome://gtdinbox/content/cross.png";
		elClose.className = "close-button";
		elClose.style.position = "absolute";
		elClose.style.right = "0px";
		elClose.style.top = "0px";
		this._eventObservers.observe(elClose, "mousedown", this.clickClose, false);
	elTitle.appendChild(elClose);
	
	this._popuper = new GtdPopuper(this._gtdinboxPage, this.el, this);
	this._popuper.addEventHandler( this, "popupOff", this.onPopupOff );	
	
	
	
	this._createLink(this.el, this._gtdinboxPage.stringBundle.getString("FilterPopup.ShowAll"), "", null, null, null, null, null, null, true );
	this._createLink(this.el, this._gtdinboxPage.stringBundle.getString("FilterPopup.Personal"), GIPROTOTYPE.restrictSearchToUser(this._gtdinboxPage._environment.emails), null, "Search", true, null, null, null, true );	
	
	if( this._type=="Contact" ){
		var email = (this._email || this._name);
		if( email.indexOf("@")>-1 ) this._createLink(this.el, this._gtdinboxPage.stringBundle.getString("FilterPopup.ComposeTo") + " " + email, "special.compose:" + email, null, null, false, true );	
	}
	
	var typesHash = {};
	typesHash[this._type] = true;
	for( t in this._parentTypes ) typesHash[t] = true;
	
	if( this._type=="Search" || this._type=="Misc" ){
		if( !typesHash["Statuses"] ) this._createStatuses();
		this._createFiles();
		this._createSearchWithin();
	}else{
		if( !typesHash["Statuses"] ) this._createStatuses();
		if( !typesHash["Contexts"] ) this._createContexts();
		if( !typesHash["Projects"] ) this._createProjects();
		if( !typesHash["Contact"] ) this._createContacts();
		if( !typesHash["References"] ) this._createReferences();
		this._createFiles();
		this._createSearchWithin();
	}
	
	var searcher = new Searcher(this._gtdinboxPage);
	searcher.showNumberResultsForEl(this.el, function(el, resultCount){	// Get the counts
			// Get the container el and strip if resultCount is 0
			if( resultCount==0 ){
				while( el.parentNode && (el.getAttribute("gtdType")!="Statuses" || !GIPROTOTYPE.hasClassName(el, "link")) ){
					el = el.parentNode;
				}
				if( el.parentNode ) el.parentNode.removeChild(el);
			}
		});	
	
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 create\n" + e.toString());
}
};

FilterPopupUI2.prototype._createBreak = function(elParent){
	if( !elParent ) elParent = this.el;
	
	var elBreak = this._doc.createElement("DIV");
		elBreak.className = "break";
	elBreak.innerHTML = "&nbsp;";
	
	elParent.appendChild( elBreak );
	
	return elBreak;
};
FilterPopupUI2.prototype._createLink = function(elParent, title, search, name, type, canExpand, excludeParentSearch, cannotPopout, replaceSearch, showNumberResults, tooltip){
try{	
	if( !excludeParentSearch ) search = this._parentSearch + " " + (search || "");
	if( replaceSearch ){	// set of replace arrays [[find,replace]]
		for( var i = 0; i < replaceSearch.length; i++ ){
			search = search.replace( replaceSearch[i][0], (replaceSearch[i][1]||"") );
		}
	}

	var elLink = this._doc.createElement("DIV");
		elLink.className = "link";
	if( elParent ) elParent.appendChild( elLink );
	
	var elBody = this._doc.createElement("DIV");
		elBody.className = "body";
		elBody.innerHTML = title;
		if( showNumberResults ) elBody.setAttribute("show-number-results", "1");
		if( tooltip ) elBody.title = tooltip;
	elLink.appendChild( elBody );
	
	var fAttribs = function(el){
		el.setAttribute("gtdSearch", search);
		if( name ) el.setAttribute("gtdName", name);
		if( type ) el.setAttribute("gtdType", type);
	};
	fAttribs(elLink);
	fAttribs(elBody);
	
	var elCmds = this._doc.createElement("DIV");
		elCmds.className = "commands";
	elLink.appendChild( elCmds );
	
	if( !cannotPopout ){
		var elPopout = this._doc.createElement("IMG");
			elPopout.className = "async-popuper";
			elPopout.src = "chrome://gtdinbox/skin/popout.png";
			elPopout.title = this._gtdinboxPage.stringBundle.getString("FilterPopup.tooltip.Open");	
			fAttribs(elPopout);
		elCmds.appendChild( elPopout );
	}
		
	if( canExpand ){ // name &&
		var elExpander = this._doc.createElement("IMG");
			elExpander.className = "expander";
			elExpander.src = "chrome://gtdinbox/content/filter-extend.png";
			elExpander.title = this._gtdinboxPage.stringBundle.getString("FilterPopup.tooltip.Drill");	
		elCmds.appendChild( elExpander );
		fAttribs(elExpander);
	}
	
	var elClear = this._doc.createElement("DIV");
		elClear.style.clear = "both";
		elClear.style.height = "1px";
	elLink.appendChild( elClear );
	
	return elLink;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 createLink\n" + e.toString());
}
};
FilterPopupUI2.prototype._createSection = function(title, elContent, elParent, show){
try{
	elParent = elParent || this.el;
	var showHider = new Showhider(this._gtdinboxPage, "filter-" + title, {doc: this._doc, 'elParent':elParent, 'title':title, 'elContent': elContent, overrideShow: show});
	this._showHiders.push(showHider);
	return showHider;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 createSection\n" + e.toString());
}
};


/* STATUSES */
FilterPopupUI2.prototype._createStatuses = function(){
try{
	this._createBreak();
		
	// Create set of links for statuses
	var labels = GIPROTOTYPE.clone(this._labels.getLabelsArray("Statuses"));
	
	var settings = this._gtdinboxPage.settings.labels.types.Statuses;
	var statusFinished = settings.prefix + settings.defaults.Finished;
	
	var elTasks = this._doc.createElement("DIV");
	
	var labelSearch = "";
	for( d in settings.defaults ){
		var label = settings.prefix + settings.defaults[d];

		for( var i = 0; i < labels.length; i++ ){
			if( labels[i]==label ){
				if( d=="Action" ){
					labelSearch = "(" + GIPROTOTYPE.searchableLabel(label) + " OR " + GIPROTOTYPE.searchableLabel(settings.prefix + settings.defaults.NextAction) + ")";
				}else{
					labelSearch = GIPROTOTYPE.searchableLabel(label);
				}
				if( d!="Finished" ) labelSearch += " -" + GIPROTOTYPE.searchableLabel(statusFinished)
				this._createLink( elTasks, label.replace(settings.prefix,""), labelSearch, label, "Statuses", true, null, null, null, true);
				labels.splice(i,1);
				i--;
			}
		}
	}
	// And the rest:
	for( var i = 0; i < labels.length; i++ ){
		var label = labels[i];
		this._createLink( elTasks, label.replace(settings.prefix,""), GIPROTOTYPE.searchableLabel(label) + " -" + GIPROTOTYPE.searchableLabel(statusFinished), label, "Statuses", true, null, null, null, true);
	}
	
	var showHider = this._createSection( this._gtdinboxPage.stringBundle.getString("FilterPopup.Statuses"), elTasks, this.el, true );
	

	
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 createStatuses\n" + e.toString());
}
};


/* PROJECTS */
FilterPopupUI2.prototype._createProjects = function(){
try{
	this._createBreak();
	
	var elProjects = this._doc.createElement("DIV");
	var showHider = this._createSection( this._gtdinboxPage.stringBundle.getString("FilterPopup.Projects"), elProjects, this.el, false);
	showHider.addEventHandler( this, "show", this._showProjects );
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 createProjects\n" + e.toString());
}	
};
FilterPopupUI2.prototype._showProjects = function(showHider){
	// Do a search that brings in all projects with actions; rank them by actions
	// If there is an Status in the names array, only use that status

	this._showProjectsContextsOrReferences(showHider, "Projects");
};


/* CONTEXTS */
FilterPopupUI2.prototype._createContexts = function(){

	this._createBreak();
	
	var elContexts = this._doc.createElement("DIV");
	var showHider = this._createSection( this._gtdinboxPage.stringBundle.getString("FilterPopup.Contexts"), elContexts, this.el, false);
	showHider.addEventHandler( this, "show", this._showContexts );
	
};
FilterPopupUI2.prototype._showContexts = function(showHider){
	// Do a search that brings in all contexts with actions; rank them by actions
	// If there is an Status in the names array, only use that status
	
	this._showProjectsContextsOrReferences(showHider, "Contexts");
};


FilterPopupUI2.prototype._showProjectsContextsOrReferences = function(showHider, type){
	// Do a search that brings in all projects with actions; rank them by actions
	// If there is an Status in the names array, only use that status
try{
	showHider.elContent.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif' class='loading'/>";
	
	var typePrefix = this._gtdinboxPage.settings.labels.types[type].prefix;

	// Create Searcher query:
	// Add parent + status restriction
	var restrictStatus = "";
	for( var i = 0; i < this._parentNames.length; i++ ){
		if( this._parentNames[i].indexOf(this._status.prefix)==0 ){
			restrictStatus = this._parentNames[i];
		}
	}
	if( this._name.indexOf(this._status.prefix)==0 ) restrictStatus = this._name;
	
	var query = this._parentSearch + " " + (restrictStatus? (GIPROTOTYPE.searchableLabel(restrictStatus) + " -" + GIPROTOTYPE.searchableLabel(this._status.Finished)) : this._status.orQuery);
	
	// Add Project restriction	
	query += " (";
	var labels = this._labels.getLabelsArray(type);
	for( var i = 0; i < labels.length; i++ ) query += GIPROTOTYPE.searchableLabel(labels[i]) + " OR ";
	query = query.replace(/ OR $/,")");
	
	
	
	var searcher = new Searcher(this._gtdinboxPage);
	searcher.search(query, GIPROTOTYPE.bind(this, function(threadList){
			try{
			
			showHider.elContent.innerHTML = "";
			var typeCollection = {};
			for( var i = 0; i < threadList._threads.length; i++ ){
				var statusActionCount = 0;
				var statusMatches = {};
				var thread = threadList._threads[i];
				for( var j = 0; j < thread.labels.length; j++ ){
					if( thread.labels[j].indexOf(this._status.prefix)==0 ){
						if( thread.labels[j]==this._status.NextAction || thread.labels[j]==this._status.Action ){
							statusActionCount++;
						}
						if( !statusMatches[thread.labels[j]] ) statusMatches[thread.labels[j]] = 0;
						statusMatches[thread.labels[j]]++;
					}
				}
				for( var j = 0; j < thread.labels.length; j++ ){
					if( thread.labels[j].indexOf(typePrefix)==0 ){
						if( !typeCollection[thread.labels[j]] ) typeCollection[thread.labels[j]] = {label:thread.labels[j], actionCount:0, statusMatches:{}, seen:0}
						typeCollection[thread.labels[j]].actionCount += statusActionCount;
						for( s in statusMatches ){
							if( !typeCollection[thread.labels[j]].statusMatches[s] ) typeCollection[thread.labels[j]].statusMatches[s] = 0;
							typeCollection[thread.labels[j]].statusMatches[s] += statusMatches[s];
							typeCollection[thread.labels[j]].seen++;
						}
					}
				}
			}
			var rank = [];
			var rankHash = {};
			for( p in typeCollection ){
				var id = GIPROTOTYPE.padZero(typeCollection[p].actionCount, 5) + p;
				rank.push(id);
				rankHash[id] = typeCollection[p];
			}
			rank.sort().reverse();
			
			if( rank.length==0 ){
				showHider.elContent.innerHTML = "<div class='no-items'>" + this._gtdinboxPage.stringBundle.getString("FilterPopup.NoItems") + "</div>";
				GIPROTOTYPE.addClassName(showHider.elTitle, "dimmed");
			}else{
				for( var i = 0; i < rank.length; i++ ){
					var data = rankHash[rank[i]];
					var title = [data.label + ": "];
					title.push( data.actionCount + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.Action") + ", " );
					title.push( (data.statusMatches[this._status.WaitingOn]||0) + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.WaitingOn") + ", " );
					title.push( (data.statusMatches[this._status.SomeDay]||0) + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.SomeDay") );
					this._createLink( showHider.elContent, data.label.replace(typePrefix,"") + " (" + data.actionCount + ")", GIPROTOTYPE.searchableLabel(data.label), data.label, type, true, null, null, null, null, title.join(""));
				}
			}
			
			}catch(e){
				if( GTDDEBUGMODE ) alert("FilterPopupUI2 _showProjectsContextsOrReferences	 callback\n" + e.toString());
			}
		}));
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 _showProjectsContextsOrReferences\n" + e.toString());
}
};

/* CONTACTS */
FilterPopupUI2.prototype._createContacts = function(){

	this._createBreak();
	
	var elContacts = this._doc.createElement("DIV");
	var showHider = this._createSection( this._gtdinboxPage.stringBundle.getString("FilterPopup.Contacts"), elContacts, this.el, false);
	showHider.addEventHandler( this, "show", this._showContacts );
	
};
FilterPopupUI2.prototype._showContacts = function(showHider){
	// Do a search that brings in all contacts with actions; rank them by actions
	// If there is an Status in the names array, only use that status
try{
	showHider.elContent.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif' class='loading'/>";
	
	
	// Create Searcher query:
	// Add parent + status restriction
	var restrictStatus = "";
	for( var i = 0; i < this._parentNames.length; i++ ){
		if( this._parentNames[i].indexOf(this._status.prefix)==0 ){
			restrictStatus = this._parentNames[i];
		}
	}
	if( this._name.indexOf(this._status.prefix)==0 ) restrictStatus = this._name;
	
	var query = this._parentSearch + " " + (restrictStatus? (GIPROTOTYPE.searchableLabel(restrictStatus) + " -" + GIPROTOTYPE.searchableLabel(this._status.Finished)) : this._status.orQuery);
	
	
	
	
	var searcher = new Searcher(this._gtdinboxPage);
	searcher.search(query, GIPROTOTYPE.bind(this, function(threadList){
			try{
			
			showHider.elContent.innerHTML = "";
			var typeCollection = {};
			for( var i = 0; i < threadList._threads.length; i++ ){
				var statusActionCount = 0;
				var statusMatches = {};
				var thread = threadList._threads[i];
				for( var j = 0; j < thread.labels.length; j++ ){
					if( thread.labels[j].indexOf(this._status.prefix)==0 ){
						if( thread.labels[j]==this._status.NextAction || thread.labels[j]==this._status.Action ){
							statusActionCount++;
						}
						if( !statusMatches[thread.labels[j]] ) statusMatches[thread.labels[j]] = 0;
						statusMatches[thread.labels[j]]++;
					}
				}
				var names = thread.getNames(null, true);
				for( var j = 0; j < names.length; j++ ){
					if( !typeCollection[names[j][1]] ) typeCollection[names[j][1]] = {name:names[j][0], email:names[j][1], actionCount:0, statusMatches:{}, seen:0}
					typeCollection[names[j][1]].actionCount += statusActionCount;
					for( s in statusMatches ){
						if( !typeCollection[names[j][1]].statusMatches[s] ) typeCollection[names[j][1]].statusMatches[s] = 0;
						typeCollection[names[j][1]].statusMatches[s] += statusMatches[s];
						typeCollection[names[j][1]].seen++;
					}
				}
			}
			var rankOwe = [];
			var rankOweHash = {};
			var rankDelegated = [];
			var rankDelegatedHash = {};
			for( p in typeCollection ){
				if( typeCollection[p].statusMatches[this._status.WaitingOn] ){
					var id = GIPROTOTYPE.padZero(typeCollection[p].statusMatches[this._status.WaitingOn], 5) + p;
					rankDelegated.push(id);
					rankDelegatedHash[id] = typeCollection[p];
				}
				if( typeCollection[p].actionCount > 0 ){
					var id = GIPROTOTYPE.padZero(typeCollection[p].actionCount, 5) + p;
					rankOwe.push(id);
					rankOweHash[id] = typeCollection[p];
				}
			}
			rankOwe.sort().reverse();
			rankDelegated.sort().reverse();
			
			
			if( rankOwe.length > 0 ){
				var elTitle = this._doc.createElement("DIV");
					elTitle.innerHTML = this._gtdinboxPage.stringBundle.getString("FilterPopup.Contacts.Owe");
					elTitle.className = "header";
				showHider.elContent.appendChild( elTitle );
				for( var i = 0; i < rankOwe.length; i++ ){
					var data = rankOweHash[rankOwe[i]];
					var title = [data.email + ": "];
					title.push( data.actionCount + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.Action") + ", " );
					title.push( (data.statusMatches[this._status.WaitingOn]||0) + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.WaitingOn") + ", " );
					title.push( (data.statusMatches[this._status.SomeDay]||0) + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.SomeDay") );
					this._createLink( showHider.elContent, data.name + " (" + data.actionCount + ")", data.email, data.name, "Contact", true, null, null, null, null, title.join(""));
				}
			}
			
			if( rankDelegated.length > 0 ){
				var elTitle = this._doc.createElement("DIV");
					elTitle.innerHTML = this._gtdinboxPage.stringBundle.getString("FilterPopup.Contacts.Delegated");
					elTitle.className = "header";
				showHider.elContent.appendChild( elTitle );
				for( var i = 0; i < rankDelegated.length; i++ ){
					var data = rankDelegatedHash[rankDelegated[i]];
					var title = [data.email + ": "];
					title.push( data.actionCount + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.Action") + ", " );
					title.push( (data.statusMatches[this._status.WaitingOn]||0) + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.WaitingOn") + ", " );
					title.push( (data.statusMatches[this._status.SomeDay]||0) + " " + this._gtdinboxPage.stringBundle.getString("FilterPopup.LineItems.SomeDay") );
					this._createLink( showHider.elContent, data.name + " (" + data.statusMatches[this._status.WaitingOn] + ")", data.email, data.name, "Contact", true, null, null, null, null, title.join(""));
				}
			}
			
			
			}catch(e){
				if( GTDDEBUGMODE ) alert("FilterPopupUI2 _showProjectsContextsOrReferences	 callback\n" + e.toString());
			}
		}));
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 _showProjectsContextsOrReferences\n" + e.toString());
}
};

/* REFERENCES */
FilterPopupUI2.prototype._createReferences = function(){

	this._createBreak();
	
	var elReferences = this._doc.createElement("DIV");
	var showHider = this._createSection( this._gtdinboxPage.stringBundle.getString("FilterPopup.References"), elReferences, this.el, false);
	showHider.addEventHandler( this, "show", this._showReferences );
	
};
FilterPopupUI2.prototype._showReferences = function(showHider){
	// Do a search that brings in all references with actions; rank them by actions
	// If there is an Status in the names array, only use that status
	
	this._showProjectsContextsOrReferences(showHider, "References");
};

/* SEARCH WITHIN */
FilterPopupUI2.prototype._createSearchWithin = function(){

	this._createBreak();
	var elSearch = this._doc.createElement("DIV");
		elSearch.className = "search";
	var elSearchInput = this._doc.createElement("INPUT");
		elSearchInput.setAttribute("type", "text");
	elSearch.appendChild( elSearchInput );
	var elSearchCmd = this._doc.createElement("BUTTON");
		elSearchCmd.innerHTML = this._gtdinboxPage.stringBundle.getString("FilterPopup.SearchButton");	
	elSearch.appendChild( elSearchCmd );
	this._createSection(this._gtdinboxPage.stringBundle.getString("FilterPopup.Search"), elSearch );	
	
};


/* FILES */

FilterPopupUI2.prototype._createFiles = function(){
	this._createBreak();
	this._createLink(this.el, this._gtdinboxPage.stringBundle.getString("FilterPopup.AssociatedFiles"), "has:attachment", null, "Search", true);	
};


FilterPopupUI2.prototype.contextMenu = function(event){
	var el = GIPROTOTYPE.element(event);
	if( el.getAttribute("gtdName") ){
		GIPROTOTYPE.stop(event);
		this.clickExtend(el);
	}
};
FilterPopupUI2.prototype.mouseDown = function(event){
	
	try{
		var el = GIPROTOTYPE.element(event);
		if( event.button==2 ) return;
		if( event.shiftKey ){	// async
			this.forceOff();
			return;
		}
		
		// Detect the type (search button, extend button, regular filter)
		if( el.nodeName=="BUTTON" ){	// search
			el = el.parentNode.getElementsByTagName("INPUT")[0];
			this._gtdinboxPage._searchBoxUI.search( this._parentSearch + " " + el.value );
			GIPROTOTYPE.stop(event);
			this.forceOff();
		}else if( el.nodeName=="IMG" && el.getAttribute("gtdName") ){	// extend
			this.clickExtend(el);
			GIPROTOTYPE.stop(event);
		}else if( el.getAttribute("gtdSearch") ){
			var search = el.getAttribute("gtdSearch");
			if( search.indexOf("special.compose:")==0 ){
				this._gtdinboxPage._composeUI.clickComposeTo( search.replace("special.compose:", "") );
			}else if( search.indexOf("special.rename:")==0 ){
				var search = search.replace("special.rename:", "");
				var splitter = search.split("|,|");
				var rename = [splitter[0], splitter[1]];	// rename from and to
				var gtdinboxPage = this._gtdinboxPage;
				var finalCallback = GIPROTOTYPE.bind(this, function(){
					switch( gtdinboxPage._gmail.getActiveViewType() ){
						case "tl":
							var el = gtdinboxPage.gmailDOM.element("inbox-link");
							GIPROTOTYPE.doMouseEvent(gtdinboxPage.doc, el, "click");	// refresh
							break;
						case "cv":	// How to do this? For the url to change? How to get the thread?
							//location.href = gtdinboxPage.gmailDOM.local("domain") + "/#inbox/" + 
					}
				});
				this._gtdinboxPage.gmailDOM.renameLabel(rename, null, finalCallback);
			}else{
				this._gtdinboxPage._searchBoxUI.search( (search||"") );
			}
			GIPROTOTYPE.stop(event);
			this.forceOff();
		}
	}catch(e){
		if( GTDDEBUGMODE ) alert("filterpopup mouseDown\n" + e.toString());
	}
};
FilterPopupUI2.prototype.clickExtend = function(el){
	// Use the line-item to generate new popup
try{
	if( this._filterChild ){	// turn off
		this._filterChild.destroy();
		this._filterChild = null;
		if( this._filterChildEl==el ) return;
	}
	var name = el.getAttribute("gtdName");
	var type = el.getAttribute("gtdType");
	var parentSearch = this._parentSearch + " " + (el.getAttribute("gtdSearch")||"");
	var parentTypes = this._parentTypes.concat( this._type );
	var parentNames = this._parentNames.concat( this._name );
	this._filterChild = new FilterPopupUI3( this._gtdinboxPage, name, type, el, parentNames, parentTypes, parentSearch, this );
	this._filterChild.popup(el,{location:'right'});
	this._filterChildEl = el;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 clickExtend\n" + e.toString());
}
};

FilterPopupUI2.prototype.addPopupChildObject = function(popupObj){
	// Add to this popuper as 'safe', and propogate up
	this._popuper.addPopupChild( popupObj );
	if( this._filterParent ) this._filterParent.addPopupChildObject( popupObj );
};

FilterPopupUI2.prototype.highlighted = function(v){
	if( typeof v=="undefined" ) return GIPROTOTYPE.hasClassName(this.el, "filter-highlighted");
	if( v ){
		GIPROTOTYPE.addClassName(this.el, "filter-highlighted");
	}else{
		GIPROTOTYPE.removeClassName( this.el, "filter-highlighted");
	}
};

FilterPopupUI2.prototype.forceOff = function(){
	// Find highest FilterPopupUI2 in chain; and destroy it
	var fp = this;
	while( fp._filterParent ) fp = fp._filterParent;
	fp.destroy();
};
FilterPopupUI2.prototype.onPopupOff = function(){
	this.destroy();
};
FilterPopupUI2.prototype.clickClose = function(){
	this.destroy();
}
FilterPopupUI2.prototype.destroy = function(){
try{
	if( !this.el ) return;	// already destroyed
	
	if( this._filterParent ) this._filterParent.highlighted(true);
	
	if( this._filterChild ){	// turn off
		this._filterChild.destroy();
	}
	
	if( this.popupOff ) this.popupOff();
	if( this._popuper ) this._popuper.destroy();
	this._eventObservers.destroy();
	
	for( var i = 0; i < this._showHiders.length; i++ ){
		this._showHiders[i].destroy();
	}
	
	this._gtdinboxPage = null;
	this._doc = null;
	
	if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
	this.el = null;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI2 destroy\n" + e.toString());
}
};










