/* Copyright Andy Mitchell 2007. No warranties */

function FilterPopupUI3(gtdinboxPage, name, type, elParentPopuper, parentNames, parentTypes, parentSearch, filterParent, lastParentSearch){
try{
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._labels = gtdinboxPage._labels;
	this._parentNames = parentNames || [];
	this._parentTypes = parentTypes || [type];
	this._parentSearch = parentSearch || "";
	this._lastParentSearch = lastParentSearch;	// Used when removeFilterParent
	if( name instanceof Array ){	// contact, [name,email]
		this._name = name[0] || name[1];
		this._email = name[1];
		//if( !this._email ) this._email = this._sp.getEmailForContactName(this._name);
	}else{
		this._name = name;
	}
	this._type = type;
	this._elParentPopuper = elParentPopuper;
	this._filterParent = filterParent;
	if( filterParent ) this._filterParent.highlighted(false);
	
	if( type=="Contact" ) this._closeContacts = new CloseContacts( gtdinboxPage );
	
		
	
	this._eventObservers = new GtdEventObservers(this);
	this._showHiders = [];
	this._threadListCustomUIs = [];
	
	this._searcher = new Searcher(this._gtdinboxPage);
	
	this._status = this._gtdinboxPage.getStatusData();
	
	this._create();
	this._eventObservers.observe( this.el, "click", this.mouseDown, true);
	this._eventObservers.observe( this.el, "contextmenu", this.contextMenu, true);
	
	this.highlighted(true);
	this.addPopupChildObject( this );
	
	
	return this;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 construct\n" + e.toString());
}
};

FilterPopupUI3.prototype._create = function(){
try{
	if( !this.el ){
		this.el = this._doc.createElement("DIV");
			this.el.className = "filter-popup";
		this._popuper = new GtdPopuper(this._gtdinboxPage, this.el, this);
		this._popuper.addEventHandler( this, "popupOff", this.onPopupOff );	
	
	}
	
	var doc = this._gtdinboxPage.gmailDOM.element("document");
	if( doc ) doc.body.appendChild( this.el );
	
	var elTitle = this._doc.createElement("DIV");
		elTitle.className = "title";
		var elPath = this._doc.createElement("DIV");
			elPath.className = "path";
		elTitle.appendChild(elPath);
		if( this._type=="Search" ){
			elPath.innerHTML = this._parentSearch;
		}else{
			if( this._parentNames.length > 0 ){
				elPath.innerHTML =  this._parentNames.join(", ");
				elPath.appendChild( this._doc.createTextNode( " [" ) );
				var elRemove = this._doc.createElement("SPAN");
					elRemove.className = "remove-parent";
					elRemove.innerHTML = this._gtdinboxPage.stringBundle.getString("FilterPopup.RemoveParent");
				elPath.appendChild( elRemove );
					this._eventObservers.observe( elRemove, "click", this.removeParentSearch, true );
				elPath.appendChild( this._doc.createTextNode( "]" ) );
			}
			if( this._type=="Contact" ){	// Load the closeness
				this._closeContacts.processContacts(null, [this._email || this._name], GIPROTOTYPE.bind(this, function(contacts, contactsHash){
						var elCloseness = this._doc.createElement("IMG");
							elCloseness.className = "closeness";
						if( contacts.length > 0 ){
							elCloseness.src = contacts[0].closenessImgSrc;
							elCloseness.title = contacts[0].closenessTooltip;
						}else{
							elCloseness.src = this._closeContacts.getZeroClosenessImg();
						}
						elTitle.insertBefore( elCloseness, elPath.nextSibling );
					}));
				
			}
			elTitle.appendChild( this._doc.createTextNode((this._name || this._email || "")) );
		}
		
	this.el.appendChild( elTitle );
	var elClose = this._doc.createElement("IMG");
		elClose.src = "chrome://gtdinbox/content/cross.png";
		elClose.className = "close-button";
		elClose.style.position = "absolute";
		elClose.style.right = "0px";
		elClose.style.top = "0px";
		this._eventObservers.observe(elClose, "mousedown", this.clickClose, false);
	elTitle.appendChild(elClose);
	
	
	this.typesHash = {};
	this.typesHash[this._type] = true;
	for( t in this._parentTypes ) this.typesHash[t] = true;
	
	this.namesHash = {};
	this.namesHash[this._name] = true;
	for( var i = 0; i < this._parentNames.length; i++ ) this.namesHash[this._parentNames[i]] = true;
	
	this._renderIntro();
	this._createBreak();
	this._renderTasks();
	this._createBreak();
	this._renderRelated();
	this._createBreak();
	this._renderFiles();
	this._createBreak();
	this._renderSearch();
	

	
	this._searcher.showNumberResultsForEl(this.el, GIPROTOTYPE.bind(this, function(el, resultCount){	// Get the counts
			// Get the container el and strip if resultCount is 0
			if( resultCount==0 ){
				while( el.parentNode && el.getAttribute("gtdType")!="Statuses" && !GIPROTOTYPE.hasClassName(el, "link") ){
					el = el.parentNode;
				}
				if( el.parentNode ) el.parentNode.removeChild(el);
			}
		}));	
	
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 create\n" + e.toString());
}
};

FilterPopupUI3.prototype._createBreak = function(elParent){
	if( !elParent ) elParent = this.el;
	
	var elBreak = this._doc.createElement("DIV");
		elBreak.className = "break";
	elBreak.innerHTML = "&nbsp;";
	
	elParent.appendChild( elBreak );
	
	return elBreak;
};
FilterPopupUI3.prototype._createLink = function(elParent, title, search, name, type, canExpand, excludeParentSearch, cannotPopout, replaceSearch, showNumberResults, tooltip, showThreads){
try{
	var fullSearch = "";
	if( !excludeParentSearch ) fullSearch = this._parentSearch + " " + (search || "");
	if( replaceSearch ){	// set of replace arrays [[find,replace]]
		for( var i = 0; i < replaceSearch.length; i++ ){
			fullSearch = fullSearch.replace( replaceSearch[i][0], (replaceSearch[i][1]||"") );
		}
	}

	var elLink = this._doc.createElement("DIV");
		elLink.className = "link";
	if( elParent ) elParent.appendChild( elLink );
	
	var elBody = this._doc.createElement("DIV");
		elBody.className = "body";
		elBody.innerHTML = title;
		if( showNumberResults ) elBody.setAttribute("show-number-results", "1");
		if( tooltip ) elBody.title = tooltip;
	elLink.appendChild( elBody );
	
	var fAttribs = function(el){
		el.setAttribute("gtdSearch", fullSearch);
		el.setAttribute("gtdSearchSegment", search);
		if( name ) el.setAttribute("gtdName", name);
		if( type ) el.setAttribute("gtdType", type);
	};
	fAttribs(elLink);
	fAttribs(elBody);
	
	var elCmds = this._doc.createElement("DIV");
		elCmds.className = "commands";
	elLink.appendChild( elCmds );
	
	if( !cannotPopout ){
		var elPopout = this._doc.createElement("IMG");
			elPopout.className = "async-popuper";
			elPopout.src = "chrome://gtdinbox/skin/popout.png";
			elPopout.title = this._gtdinboxPage.stringBundle.getString("FilterPopup.Tooltip.Open");	
			fAttribs(elPopout);
		elCmds.appendChild( elPopout );
	}
		
	if( canExpand ){ // name &&
		var elExpander = this._doc.createElement("IMG");
			elExpander.className = "expander";
			elExpander.src = "chrome://gtdinbox/content/filter-extend.png";
			elExpander.title = this._gtdinboxPage.stringBundle.getString("FilterPopup.Tooltip.Drill");	
		elCmds.appendChild( elExpander );
		fAttribs(elExpander);
	}
	
	if( showThreads ){
		var elShowThreads = this._doc.createElement("IMG");
			elShowThreads.className = "show-threads";
			elShowThreads.src = "chrome://gtdinbox/content/show-threads.png";
			elShowThreads.title = this._gtdinboxPage.stringBundle.getString("FilterPopup.Tooltip.ShowThreads");	
		elCmds.appendChild( elShowThreads );
		fAttribs(elShowThreads);
		
		var elPreview = this._doc.createElement("DIV");
			elPreview.className = "show-threads-preview-box";
			elPreview.style.display = "none";
		elParent.appendChild( elPreview );
	}
	
	var elClear = this._doc.createElement("DIV");
		elClear.style.clear = "both";
		elClear.style.height = "1px";
	elLink.appendChild( elClear );
	
	return elLink;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 createLink\n" + e.toString());
}
};
FilterPopupUI3.prototype._createSection = function(title, elContent, elParent, show){
try{
	elParent = elParent || this.el;
	//var showHider = new Showhider(this._gtdinboxPage, "filter-" + title, {doc: this._doc, 'elParent':elParent, 'title':title, 'elContent': elContent, overrideShow: show});
	//this._showHiders.push(showHider);
	//return showHider;
	
	var section = null;
	var el = this._doc.createElement("DIV");
	el.className = "section";
	var elTitle = this._doc.createElement("DIV");
		elTitle.innerHTML = title;
		elTitle.className = "title";
	if( !elContent ){
		elContent = this._doc.createElement("DIV");
		elContent.className = "content";
	}
	el.appendChild( elTitle );
	el.appendChild( elContent );
	
	if( elParent ) elParent.appendChild( el );
	section = {'el':el, 'elTitle':elTitle, 'elContent':elContent};
	
	return section;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 createSection\n" + e.toString());
}
};







FilterPopupUI3.prototype._renderIntro = function(){

try{
	this._createLink(this.el, this._gtdinboxPage.stringBundle.getString("FilterPopup.ShowAll"), "", null, null, null, null, null, null, true, null, true );
	
	if( this._type=="Projects" || this._type=="Statuses" || this._type=="Contexts" ){
		this._createLink(this.el, this._gtdinboxPage.stringBundle.getString("FilterPopup.Personal"), GIPROTOTYPE.restrictSearchToUser(this._gtdinboxPage._environment.emails), null, "Search", null, null, null, null, true, null, true );	
	}
	
	if( this._type=="Contact" ){
		var email = (this._email || this._name);
		if( email.indexOf("@")>-1 ) this._createLink(this.el, this._gtdinboxPage.stringBundle.getString("FilterPopup.ComposeTo") + " " + email, "special.compose:" + email, null, null, false, true );	
	}
	

}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 renderIntro\n" + e.toString());
}
};

FilterPopupUI3.prototype._renderTasks = function(){
	// Create a tab box
	// Choose what to add to it based on the type
	// Load the first tab; then load the other tabs
	
try{
	var showHider = this._createSection( this._gtdinboxPage.stringBundle.getString("FilterPopup.Tasks") , null, null, true );
	
	if( this._type=="Contact" ){	// Render special screen - broken into WaitingOn and everything else
		this._searcher.search( this._getQueryByType("Contact"), GIPROTOTYPE.bind(this, function(threadList){
			var fAddTitle = function(type, elTitle){ showHider.elContent.appendChild( elTitle ) };
			var fAddLineItem = GIPROTOTYPE.bind(this, function(type, name, displayName, title, search, bucket){ this._createLink(showHider.elContent, displayName, search, name, type, true, null, null, null, null, title ) });
			this._searcher.renderThreadListByType("Contact", threadList, "tasks", fAddTitle, fAddLineItem, function(type, count){
					if( showHider.elContent.innerHTML=="" ){	// Nothing found
						showHider.elContent.style.display = "none";
						GIPROTOTYPE.addClassName( showHider.elTitle, "title-disabled" );
					}
				}, this.namesHash, null, true );
			
		}));
	}else{
		var tabs = null;
		if( this._type=="Projects" ) tabs = ["Statuses", "Contact"];
		if( this._type=="Contexts" ) tabs = ["Statuses", "Contact", "Projects"];	
		if( this._type=="Reference" ) tabs = ["Statuses", "Contact", "Projects", "Contexts"];
		
		if( tabs ){
			var processedTabs = 0;
			var tabBox = new TabBox(this._doc, "tasks", {elParent:showHider.elContent, gtdinboxPage:this._gtdinboxPage} );
			tabBox.loading(true); 
			for( var i = 0; i < tabs.length; i++ ) tabBox.add(tabs[i], this._gtdinboxPage.stringBundle.getString("FilterPopup.Tabs." + tabs[i]) );
			
			// Render the first:
			
			this._searcher.search( this._getQueryByType(tabs[0], true), GIPROTOTYPE.bind(this, function(threadList){
					var fAddTitle = function(type, elTitle){ tabBox.contentEl(type).appendChild( elTitle ) };
					var fAddLineItem = GIPROTOTYPE.bind(this, function(type, name, displayName, title, search, bucket){ this._createLink(tabBox.contentEl(type), displayName, search,name, type, true, null, null, null, null, title ) });
					var fCallback = function(type, count){
						tabBox.title( type, tabBox.title(type) + " (" + count + ")" );
						if( count==0 ) tabBox.visible( type, false );
						processedTabs++;							
						if( tabs.length==processedTabs ){
							tabBox.loading(false);
							if( tabBox.numberOfActiveTabs()==0 ){
								showHider.elContent.style.display = "none";
								GIPROTOTYPE.addClassName( showHider.elTitle, "title-disabled" );
							}
						}
					}
					//this._renderQueryByType(tabs[0], tabBox.contentEl(tabs[0]), threadList, true, );
					this._searcher.renderThreadListByType(tabs[0], threadList, "tasks", fAddTitle, fAddLineItem, fCallback, this.namesHash, this._status, true);
										
					// Now load the rest:
					var queryToName = {};
					for( var i = 1; i < tabs.length; i++ ){
						var query = this._getQueryByType(tabs[i], true);
						queryToName[query] = tabs[i];
						this._searcher.search( query, GIPROTOTYPE.bind(this, function(threadList, query){
							//this._renderQueryByType(name, tabBox.contentEl(type), threadList, true, );
							this._searcher.renderThreadListByType(queryToName[query], threadList, "tasks", fAddTitle, fAddLineItem, fCallback, this.namesHash, this._status, true);
						}));
					}
				}));
		}
	}


}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 renderTasks\n" + e.toString());
}
};





FilterPopupUI3.prototype._renderRelated = function(){
	// Create a tab box
	// Choose what to add to it based on the type
	// Load the first tab; then load the other tabs
	
try{
	var showHider = this._createSection( this._gtdinboxPage.stringBundle.getString("FilterPopup.Related") , null, null, true );
	

		var tabs = null;
		if( this._type=="Contact" ) tabs = ["Projects", "Contexts", "References"];
		if( this._type=="Projects" ) tabs = ["Contact", "Projects", "Contexts", "References"];
		if( this._type=="Contexts" ) tabs = ["Contact", "Projects", "References"];	
		if( this._type=="Reference" ) tabs = ["Contacts", "Projects"];
		
		if( tabs ){
			var processedTabs = 0;
			var tabBox = new TabBox(this._doc, "tasks", {elParent:showHider.elContent, gtdinboxPage:this._gtdinboxPage} );
			tabBox.loading(true);
			for( var i = 0; i < tabs.length; i++ ) tabBox.add(tabs[i], this._gtdinboxPage.stringBundle.getString("FilterPopup.Tabs." + tabs[i]) );
			
			// Render the first:
			
			this._searcher.search( this._getQueryByType(tabs[0]), GIPROTOTYPE.bind(this, function(threadList){
					var fAddTitle = function(type, elTitle){ tabBox.contentEl(type).appendChild( elTitle ) };
					var fAddLineItem = GIPROTOTYPE.bind(this, function(type, name, displayName, title, search, bucket){ this._createLink(tabBox.contentEl(type), displayName, search, name, type, true, null, null, null, null, title ) });
					var fCallback = function(type, count){
						tabBox.title( type, tabBox.title(type) + " (" + count + ")" );
						if( count==0 ) tabBox.visible( type, false );
						processedTabs++;
						if( tabs.length==processedTabs ){
							tabBox.loading(false);
							if( tabBox.numberOfActiveTabs()==0 ){
								showHider.elContent.style.display = "none";
								GIPROTOTYPE.addClassName( showHider.elTitle, "title-disabled" );
							}
						}
					}
					//this._renderQueryByType(tabs[0], tabBox.contentEl(tabs[0]), threadList, false, );
					this._searcher.renderThreadListByType(tabs[0], threadList, "volume", fAddTitle, fAddLineItem, fCallback, this.namesHash, this._status);
					
					// Now load the rest:
					var queryToName = {};
					for( var i = 1; i < tabs.length; i++ ){
						var query = this._getQueryByType(tabs[i],true);
						queryToName[query] = tabs[i];
						this._searcher.search( query, GIPROTOTYPE.bind(this, function(threadList, query){
							//var type = queryToName[query];
							this._searcher.renderThreadListByType(queryToName[query], threadList, "volume", fAddTitle, fAddLineItem, fCallback, this.namesHash, this._status);
						}));
					}
				}));
		}



}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 renderTasks\n" + e.toString());
}
};




FilterPopupUI3.prototype._getQueryByType = function(type, incStatuses){
try{
	var query = this._parentSearch;
	if( incStatuses ){
		query += " (";
		var statuses = this._labels.getLabelsArray("Statuses");
		for( var i = 0; i < statuses.length; i++ ) query += GIPROTOTYPE.searchableLabel( statuses[i] ) + " OR ";
		query = query.replace(/ OR $/, ")");
	}
		
	switch(type){
		case "Statuses":	// All statuses
			break;
		case "Contact":	// All statuses
			break;
		case "Projects": // All projects and statuses
			var projects = this._labels.getLabelsArray("Projects");
			if( projects.length > 0 ){
				query += " (";
				for( var i = 0; i < projects.length; i++ ) query += GIPROTOTYPE.searchableLabel( projects[i] ) + " OR ";
				query = query.replace(/ OR $/, ")");
			}
			break;
		case "Contexts": // All contexts and statuses
			var contexts = this._labels.getLabelsArray("Contexts");
			if( contexts.length > 0 ){
				query += " (";
				for( var i = 0; i < contexts.length; i++ ) query += GIPROTOTYPE.searchableLabel( contexts[i] ) + " OR ";
				query = query.replace(/ OR $/, ")");
			}
			break;
	}
	return query;
	

}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 getQueryByType\n" + e.toString());
}
};









/* SEARCH WITHIN */
FilterPopupUI3.prototype._renderSearch = function(){

	var elSearch = this._doc.createElement("DIV");
		elSearch.className = "search";
	var elSearchInput = this._doc.createElement("INPUT");
		elSearchInput.setAttribute("type", "text");
	elSearch.appendChild( elSearchInput );
	var elSearchCmd = this._doc.createElement("BUTTON");
		elSearchCmd.innerHTML = this._gtdinboxPage.stringBundle.getString("FilterPopup.SearchButton");	
	elSearch.appendChild( elSearchCmd );
	this._createSection(this._gtdinboxPage.stringBundle.getString("FilterPopup.Search"), elSearch );	
	
};


/* FILES */

FilterPopupUI3.prototype._renderFiles = function(){
	var showHider = this._createSection( this._gtdinboxPage.stringBundle.getString("FilterPopup.AssociatedFiles") , null, null, true );
	this._filesUI = new FilesUI(this._gtdinboxPage);
	showHider.elContent.appendChild(this._filesUI.el);
	this._filesUI.load( this._parentSearch + " has:attachment" );
};






































FilterPopupUI3.prototype.contextMenu = function(event){
	var el = GIPROTOTYPE.element(event);
	if( el.getAttribute("gtdName") ){
		GIPROTOTYPE.stop(event);
		this.clickExtend(el);
	}
};
FilterPopupUI3.prototype.mouseDown = function(event){
	
	try{
		var el = GIPROTOTYPE.element(event);
		if( event.button==2 ) return;
		if( event.shiftKey ){	// async
			this.forceOff();
			return;
		}
		
		
		// Detect the type (search button, extend button, regular filter)
		if( el.nodeName=="BUTTON" ){	// search
			el = el.parentNode.getElementsByTagName("INPUT")[0];
			this._gtdinboxPage._searchBoxUI.search( this._parentSearch + " " + el.value );
			GIPROTOTYPE.stop(event);
			this.forceOff();
		}else if( el.nodeName=="IMG" && el.className=="show-threads" ){
			GIPROTOTYPE.stop(event);
			var elLink = el.parentNode;
			while( elLink.parentNode && elLink.className.indexOf("link")==-1 ){
				elLink = elLink.parentNode;
			}
			if( elLink.parentNode ){
				var elPreview = elLink.nextSibling;
				while( elPreview.nextSibling && elPreview.className.indexOf("show-threads-preview-box")==-1 ){
					elPreview = elPreview.nextSibling;
				}
				if( elPreview.className.indexOf("show-threads-preview-box")>-1 ){
					if( elPreview.innerHTML=="" ){	// Load it
						this._searcher.search( elLink.getAttribute("gtdSearch"), GIPROTOTYPE.bind(this, function(threadList){
							if( threadList._threads.length > 0 ){
								var threadListCustomUI = new ThreadListCustomUI(this._gtdinboxPage );
								threadListCustomUI.load( threadList, null, {excludeNames:true, excludeDate:true, excludeLabels:true, tooltipNames:true, tooltipDate:true} );
								this._threadListCustomUIs.push( threadListCustomUI );
								elPreview.appendChild(threadListCustomUI.el);
							}
						}));
					}
					if( elPreview.style.display=="none" ){
						elPreview.style.display = "";
					}else{
						elPreview.style.display = "none";
					}
				}
			}
		}else if( el.nodeName=="IMG" && el.getAttribute("gtdName") ){	// extend
			this.clickExtend(el);
			GIPROTOTYPE.stop(event);
		}else if( el.getAttribute("gtdSearch") ){
			var search = el.getAttribute("gtdSearch");
			if( search.indexOf("special.compose:")==0 ){
				this._gtdinboxPage._composeUI.clickComposeTo( search.replace("special.compose:", "") );
			}else if( search.indexOf("special.rename:")==0 ){
				var search = search.replace("special.rename:", "");
				var splitter = search.split("|,|");
				var rename = [splitter[0], splitter[1]];	// rename from and to
				var gtdinboxPage = this._gtdinboxPage;
				var finalCallback = GIPROTOTYPE.bind(this, function(){
					switch( gtdinboxPage._gmail.getActiveViewType() ){
						case "tl":
							var el = gtdinboxPage.gmailDOM.element("inbox-link");
							GIPROTOTYPE.doMouseEvent(gtdinboxPage.doc, el, "click");	// refresh
							break;
						case "cv":	// How to do this? For the url to change? How to get the thread?
							//location.href = gtdinboxPage.gmailDOM.local("domain") + "/#inbox/" + 
					}
				});
				this._gtdinboxPage.gmailDOM.renameLabel(rename, null, finalCallback);
			}else{
				this._gtdinboxPage._searchBoxUI.search( (search||"") );
			}
			GIPROTOTYPE.stop(event);
			this.forceOff();
		}
	}catch(e){
		if( GTDDEBUGMODE ) alert("filterpopup mouseDown\n" + e.toString());
	}
};
FilterPopupUI3.prototype.clickExtend = function(el){
	// Use the line-item to generate new popup
try{
	if( this._filterChild ){	// turn off
		this._filterChild.destroy();
		this._filterChild = null;
		if( this._filterChildEl==el ) return;
	}
	var name = el.getAttribute("gtdName");
	var type = el.getAttribute("gtdType");
	var parentSearch = (el.getAttribute("gtdSearch")||"");
	var search = (el.getAttribute("gtdSearchSegment")||"");
	var parentTypes = this._parentTypes.concat( this._type );
	var parentNames = this._parentNames.concat( this._name );
	this._filterChild = new FilterPopupUI3( this._gtdinboxPage, name, type, el, parentNames, parentTypes, parentSearch, this, search );
	this._filterChild.popup(el,{location:'right'});
	this._filterChildEl = el;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 clickExtend\n" + e.toString());
}
};

FilterPopupUI3.prototype.addPopupChildObject = function(popupObj){
	// Add to this popuper as 'safe', and propogate up
	this._popuper.addPopupChild( popupObj );
	if( this._filterParent ) this._filterParent.addPopupChildObject( popupObj );
};

FilterPopupUI3.prototype.highlighted = function(v){
	if( typeof v=="undefined" ) return GIPROTOTYPE.hasClassName(this.el, "filter-highlighted");
	if( v ){
		GIPROTOTYPE.addClassName(this.el, "filter-highlighted");
	}else{
		GIPROTOTYPE.removeClassName( this.el, "filter-highlighted");
	}
};

FilterPopupUI3.prototype.removeParentSearch = function(){
try{
	// Kill parents
	var parent = this;
	while( parent._filterParent ){
		parent = parent._filterParent;
		parent.destroy(true);
	}
	
	// Remove all gumph:
	this._parentNames = [];
	this._parentTypes = [this._type];
	this._parentSearch = this._lastParentSearch;
	this._elParentPopuper = null;
	this._filterParent = null;
	this.el.innerHTML = "";

	// kill showhiders
	for( var i = 0; i < this._showHiders.length; i++ ){
		this._showHiders[i].destroy();
	}
	for( var i = 0; i < this._threadListCustomUIs.length; i++ ){
		this._threadListCustomUIs[i].destroy();
	}
	
	// Rebuild
	this._create();
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 removeParentSearch\n" + e.toString());
}
};

FilterPopupUI3.prototype.forceOff = function(){
	// Find highest FilterPopupUI3 in chain; and destroy it
	var fp = this;
	while( fp._filterParent ) fp = fp._filterParent;
	fp.destroy();
};
FilterPopupUI3.prototype.onPopupOff = function(){
	this.destroy();
};
FilterPopupUI3.prototype.clickClose = function(){
	this.destroy();
}
FilterPopupUI3.prototype.destroy = function(ignoreKin){
try{
	if( !this.el ) return;	// already destroyed
		
	if( !ignoreKin && this._filterParent ){
		this._filterParent.highlighted(true);
	}
	
	if( !ignoreKin && this._filterChild ){	// turn off
		this._filterChild.destroy();
	}
	
	if( this._popuper ){
		this._popuper.resetDeadZones(); // ensure doesn't fire and close me again
		this._popuper.resetEventHandlers();	// ensure doesn't fire and close me again
	}
	
	if( this.popupOff ) this.popupOff();
	if( this._popuper ) this._popuper.destroy();
	this._eventObservers.destroy();
	
	for( var i = 0; i < this._showHiders.length; i++ ){
		this._showHiders[i].destroy();
	}
	for( var i = 0; i < this._threadListCustomUIs.length; i++ ){
		this._threadListCustomUIs[i].destroy();
	}
	
	if( this._closeContacts ) this._closeContacts.destroy();
	if( this._searcher ) this._searcher.destroy();
	if( this._filesUI ) this._filesUI.destroy();
	
	this._gtdinboxPage = null;
	this._doc = null;
	
	if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
	this.el = null;
}catch(e){
	if( GTDDEBUGMODE ) alert("FilterPopupUI3 destroy\n" + e.toString());
}
};










