function LabelsBoxCount(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	
	this._labelCounts = {};
	
	
	this._multiToOne = new MultiToOne(gtdinboxPage, 4000);
	
	// Handle manual thread changes:
	var fRefresh = function(obj, labelname){
			setTimeout( GIPROTOTYPE.bind(this, function(){
				this.refresh(false, labelname)
			}), 300 );
		};
	this._gtdinboxPage._threadList.addEventHandler(this, "addThreadLabel", fRefresh );
	this._gtdinboxPage._threadList.addEventHandler(this, "removeThreadLabel", fRefresh );
	
};


LabelsBoxCount.prototype.refresh = function(full, labelname, forceRun){
try{
	// Find each label
	// If changed since last processed, update
		
	var found = false;
	for( t in this._gtdinboxPage.settings.ui.showCounts ) if( this._gtdinboxPage.settings.ui.showCounts[t] ) found = true;
	if( !found ) return; // Make sure there is something to do

	this._gtdinboxPage._perfLogger.countCall("LabelsBoxCount.refresh");

	// Try to queue it:
	if( this._runOnce && !forceRun ){
		this._multiToOne.process("refresh", GIPROTOTYPE.bind(this, function(){
			this.refresh(full, labelname, true)}));
		return;
	}
	this._runOnce = true;
	
	this._gtdinboxPage._perfLogger.countCall("LabelsBoxCount.refresh.inprogress");
	this._gtdinboxPage._perfLogger.log("LabelsBoxCount.refresh", "start", true);

	// Do not allow full refreshes often


	// Lazy load:
	if( !this._searcher ) this._searcher = new Searcher(this._gtdinboxPage)
	
	
	// Try to clear out existing items
	//if( full ) this._labelHash = {};	// force clear out
	//if( labelname && this._labelHash[labelname] ) this._labelHash = {}; // has to full clear, as projects/contexts might use the status label //delete this._labelHash[labelname] // clear out just a label
	
	// Preform useful strings:
	var statusFinished = this._gtdinboxPage.settings.labels.types.Statuses.prefix + this._gtdinboxPage.settings.labels.types.Statuses.defaults.Finished;
	
	// Form the query. All active, unfinished, labels we are allowed to process.
	var q = "(";
	var arr = this._gtdinboxPage._labels.getLabelsArray("Statuses");
	var j = 0;
	for( var i = 0; i < arr.length; i++ ){
		if( arr[i]!=statusFinished ){
			if( j > 0 ) q += " OR ";
			q += GIPROTOTYPE.searchableLabel(arr[i]);
			j++;
		}
	}
	q += ") -" + GIPROTOTYPE.searchableLabel(statusFinished);
	
	
	// Process each label:
	var labels = {};
	var changedCount = false;	// Used to see if a count has changed (e.g. a mark read causing a non full/labelname update)
	if( !this._result || full ) this._result = this._gtdinboxPage.gmailDOM.elements("label-names");
	for( var i = 0; i < this._result.snapshotLength; i++ ){
		// Extract the label:
		var elName = this._result.snapshotItem(i);
		var type = elName.getAttribute("gtdLabelType");
		var label = elName.getAttribute("gtdLabelName");
		
		if( label!=statusFinished ){
			if( this._gtdinboxPage.settings.ui.showCounts[type]  ){
				labels[label] = true;
				if( labels[label] ) labels[label] = {'label':label, 'type':type, 'elName':elName, 'count':0};
			
				if( this._extractLabelCount( elName, true )!=this._labelCounts[label] ){
					/*
					this._gtdinboxPage.unsafeWindow.console.log("changed count: " + label);
					this._gtdinboxPage.unsafeWindow.console.log( this._extractLabelCount( elName, true ) );
					this._gtdinboxPage.unsafeWindow.console.log( this._labelCounts[label] );
					*/
					changedCount = true;
				}
			}
		}
	}
	
	q += " (";
	var j = 0;
	for( l in labels ){
		if( labels[l].label!=statusFinished ){
			if( j > 0 ) q += " OR ";
			q += GIPROTOTYPE.searchableLabel(labels[l].label);
			j++;
		}
	}
	q += ")";
	
	
	this._gtdinboxPage._perfLogger.log("LabelsBoxCount.refresh", "check labels box");
	
	
	if( !changedCount && !full && !labelname ){
		return;	// No point proceeding. Nothing changed.
	}
	
	
	this._gtdinboxPage._perfLogger.countCall("LabelsBoxCount.refresh.fullCallback");
	
	// Using the full query, load all threads
	this._searcher.search(q, GIPROTOTYPE.bind(this, function(threadList){
		
		for( var i = 0; i < threadList._threads.length; i++ ){
			var thread = threadList._threads[i];
			for( var j = 0; j < thread.labels.length; j++ ){
				if( labels[thread.labels[j]] ) labels[thread.labels[j]].count++;
			}
		}
		
		
		// Now render the labels
		for( l in labels ){
			var label = labels[l];
			if( labels[l].type=="Statuses" ){
				if( label.count > 0 ){
					label.elName.innerHTML = label.elName.innerHTML.replace(/(.+?)(\s(\(|\[)\d+(\)|\]))?$/, "$1") + " [" + label.count + "]";
					label.elName.style.fontWeight = "bold";
				}else{
					label.elName.innerHTML = label.elName.innerHTML.replace(/(.+?)(\s(\(|\[)\d+(\)|\]))?$/, "$1");
					label.elName.style.fontWeight = "normal";
				}
				label.elName.title = label.elName.innerHTML;
				this._labelCounts[label.label] = label.count;
			}else{
				var unread = this._extractLabelCount(label.elName);
				var totalCount = label.count + unread;
			
				if( totalCount > 0 ){
					label.elName.innerHTML = label.elName.innerHTML.replace(/(.+?)(\s(\(|\[)\d+(\)|\]))?$/, "$1") + " [" + totalCount + "]";
					label.elName.style.fontWeight = "bold";
				}else{
					label.elName.innerHTML = label.elName.innerHTML.replace(/(.+?)(\s(\(|\[)\d+(\)|\]))?$/, "$1");
					label.elName.style.fontWeight = "normal";
				}
				label.elName.title = l + " (" + unread + " unread, " + label.count + " with a Status)";
				label.elName.setAttribute("gtd-labelsboxcount-unread", unread);
				this._labelCounts[label.label] = totalCount;
			}
		}
		
		this._gtdinboxPage._perfLogger.log("LabelsBoxCount.refresh", "end of callback");
			
	}));
	

}catch(e){
	if( GTDDEBUGMODE ) alert("LabelsBoxCount refresh\n" + e.toString());
}
};


/*



		if( this._gtdinboxPage.settings.ui.showCounts.Statuses && type=="Statuses" && label!=statusFinished  ){	// Process Statuses
		
			if( this._labelHash[label] ){	// Exists
				this._updateStatusLabel( this._labelHash[label] );
			}else{
				if( !this._labelHash[label] ) this._labelHash[label] = {'label':label, 'elName':null};
				
				// Generate the query:
				var q = GIPROTOTYPE.searchableLabel(label);
				if( label!=statusFinished ) q += " -" + GIPROTOTYPE.searchableLabel(statusFinished);
				

				this._queryHash[q] = {'label':label};
				this._labelHash[label].elName = elName;

				this._searcher.getNumberResults(q, GIPROTOTYPE.bind(this, function(query, count){
						this._gtdinboxPage.unsafeWindow.console.log("callback: " + query);
						var label = this._queryHash[query].label;
						var l = this._labelHash[ label ];
						this._updateStatusLabel(l, count);
					}));
				
			}
		}else if( this._gtdinboxPage.settings.ui.showCounts[type] && label!=statusFinished  ){	// Process Rest
			
			// If not cached or the count has changed, reload it:
			if( !this._labelHash[label] || (this._extractLabelCount(this._labelHash[label].elName.innerHTML)!=this._labelHash[label].totalCount) ){
				
				if( !this._labelHash[label] ) this._labelHash[label] = {'label':label, 'elName':null};
				
				if( !this._labelHash[label].elName ) this._gtdinboxPage.unsafeWindow.console.log("did not find " + label);
				else this._gtdinboxPage.unsafeWindow.console.log("loading label '" + label + "', extracted count: " + this._extractLabelCount(this._labelHash[label].elName.innerHTML) + ", totalCount: " + this._labelHash[label].totalCount);
				
				// Generate the query:
				var q = statuses + " " + GIPROTOTYPE.searchableLabel(label);
				this._queryHash[q] = {'label':label};
				
				this._labelHash[label].elName = elName;

				this._searcher.getNumberResults(q, GIPROTOTYPE.bind(this, function(query, count){
						this._gtdinboxPage.unsafeWindow.console.log("callback: " + query);
						var label = this._queryHash[query].label;
						var l = this._labelHash[ label ];
						
						this._updateLabel( l, count );
					}));
			}else{
				this._updateLabel( this._labelHash[label] );
			}
		}
	
	}

LabelsBoxCount.prototype._updateStatusLabel = function(l, count){
	if( typeof count=="undefined" ) count = l.count;
	if( count > 0 ){
		l.elName.innerHTML = l.elName.innerHTML.replace(/(.+?)(\s\(\d+\))?$/, "$1") + " (" + count + ")";
		l.elName.style.fontWeight = "bold";
	}else{
		l.elName.innerHTML = l.elName.innerHTML.replace(/(.+?)(\s\(\d+\))?$/, "$1");
		l.elName.style.fontWeight = "normal";
	}
	l.count = count;
	l.elName.title = l.elName.innerHTML;
};

LabelsBoxCount.prototype._updateLabel = function(l, count){
	// Extract the unread value:
	
	var unread = 0;
	var totalCount = 0;
	if( typeof count=="undefined" ){	// read from memory
		count = l.count;
		unread = l.unread;
		totalCount = l.totalCount;
	}else{
		unread = this._extractLabelCount(l.elName.innerHTML);
		this._gtdinboxPage.unsafeWindow.console.log("unread for '" + l.elName.innerHTML + "': " + unread);
		totalCount = count + unread;
	}
	
	if( totalCount > 0 ){
		l.elName.innerHTML = l.elName.innerHTML.replace(/(.+?)(\s\(\d+\))?$/, "$1") + " (" + totalCount + ")";
		l.elName.style.fontWeight = "bold";
	}else{
		l.elName.innerHTML = l.elName.innerHTML.replace(/(.+?)(\s\(\d+\))?$/, "$1");
		l.elName.style.fontWeight = "normal";
	}
	l.totalCount = totalCount;
	l.count = count;
	l.unread = unread;
	l.elName.title = l.label + " (" + unread + " unread, " + count + " with a Status)";
};

*/

LabelsBoxCount.prototype._extractLabelCount = function(elName, forceRead){
	var html = elName.innerHTML;
	if( !forceRead && /.+?\s\[(\d+)\]$/.test( html ) ){	// changed by us
		return parseInt( elName.getAttribute("gtd-labelsboxcount-unread") );
	}else{
		var m = html.match( /.+?\s\[(\d+)\]$/ );
		if( m && m[1] ) return parseInt(m[1]);
		var m = html.match( /.+?\s\((\d+)\)$/ );
		if( m && m[1] ) return parseInt(m[1]);
		return 0;
	}
};