function ModifyReviews(gtdinboxPage, el, lightbox){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this.el = el;
	this.elModifyReviews = el.ownerDocument.getElementById("gtd.modifyreviews");
		GIPROTOTYPE.addClassName( this.elModifyReviews, "modify-reviews" );
	this.elCreate = el.ownerDocument.getElementById("gtd.newreview.create");
		this.elCreateTitle = el.ownerDocument.getElementById("gtd.newreview.title");
		this.elCreateCategory = el.ownerDocument.getElementById("gtd.newreview.category");
		this.elCreateSearch = el.ownerDocument.getElementById("gtd.newreview.searchquery");
	
	this._eventObservers = new GtdEventObservers(this);
	
	this._eventObservers.observe(this.elModifyReviews, "click", this._click, true);
	this._eventObservers.observe(this.elCreate, "click", this._clickCreate, true);
	
	this.refresh();
}

ModifyReviews.prototype.refresh = function(secondPass){
	// Rebuild the list of Reviews

try{
	this.elModifyReviews.innerHTML = "";
	
	var statusPrefix = this._gtdinboxPage.settings.labels.types.Statuses.prefix;
	var statusDefaults = this._gtdinboxPage.settings.labels.types.Statuses.defaults;

	var duplicateFinder = {};
	var duplicates = {};
	var foundDuplicates = false;
	
	var reviews = this._gtdinboxPage.settings.reviews;
	if( reviews ){
		var categories = {};
		
		for( var i = 0; i < reviews.length; i++ ){
			
			var title = reviews[i].title || "";
			if( !title && reviews[i].titleKey ) title = this._gtdinboxPage.stringBundle.getString(reviews[i].titleKey);
			
			var category = this._getCategory(reviews[i]);
			
			var search = reviews[i].search || "";
			
			if( title ){
				var firstInCategory = false;
				if( !categories[category] ){
					firstInCategory = true;
					categories[category] = this._addTitle(category);
				}
				categories[category] = this._addItem(reviews[i].id, title, search, categories[category].nextSibling, null, firstInCategory);
				
				if( duplicateFinder[(title + category + search).toString()] || duplicateFinder[reviews[i].id] ){
					duplicates[reviews[i].id] = true;
					foundDuplicates = true;
				}else{
					duplicateFinder[(title + category + search).toString()] = true;
					duplicateFinder[reviews[i].id] = true;
				}
			}else{	// Should not exist
				duplicates[reviews[i].id] = true;
			}
		}		
	}

	if( foundDuplicates && !secondPass ){
		
		for( var i = this._gtdinboxPage.settings.reviews.length - 1; i >= 0; i-- ){
			var id = this._gtdinboxPage.settings.reviews[i].id;
			if( duplicates[id] ){
				this._gtdinboxPage.settings.reviews.splice(i,1);
				duplicates[id] = false;
			}
		}
		
		this._gtdinboxPage.saveSettings();
		this._gtdinboxPage._dashboard.refresh();
		this.refresh(true);
	}
	
	// Remove 'Down' option from all last items in category
	for( c in categories ){
		var el = GIPROTOTYPE.getElementsByClassName(categories[c], "move-down-holder")[0];
		el.parentNode.removeChild(el);
		el = GIPROTOTYPE.getElementsByClassName(categories[c], "move-holder")[0];
		if( GIPROTOTYPE.trim(el.innerHTML)=="[]" ) el.parentNode.removeChild(el); 
	}

}catch(e){
	if( GTDDEBUGMODE ) alert("ModifyReviews refresh\n" + e.toString());
}
};

ModifyReviews.prototype._getCategory = function(review){
	var category = review.category || "";
	if( !category && review.categoryKey ) category = this._gtdinboxPage.stringBundle.getString(review.categoryKey);
	if( !category ) category = this._gtdinboxPage.stringBundle.getString("Dashboard.Reviews.Misc");
	return category;
};

ModifyReviews.prototype._addTitle = function(title){
try{
	var el = this._doc.createElement("DIV");
	el.className = "title";
	el.innerHTML = title;
	el.setAttribute("category", title);
	
	var elCmds = this._doc.createElement("SPAN");
		elCmds.className = "commands";
		var html = [];
		html.push( " (" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.Move") );
		html.push( " <a href='#' class='title-move-up'>" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.MoveUp") + "</a>" );
		html.push( ", <a href='#' class='title-move-down'>" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.MoveDown") + "</a>)" );
		elCmds.innerHTML = html.join("");
	el.appendChild( elCmds );
	
	this.elModifyReviews.appendChild( el );
	
	return el;
}catch(e){
	if( GTDDEBUGMODE ) alert("ModifyReviews _addTitle\n" + e.toString());
}
};

ModifyReviews.prototype._addItem = function(id, title, search, insertBefore, inbuilt, firstInCategory){

try{
	var el = this._doc.createElement("DIV");
		el.className = "line-item";
		
	search = search.replace( /</g, "&lt;" );
	search = search.replace( />/g, "&gt;" );
	
	var elName = this._doc.createElement("DIV");
		elName.className = "name";
		elName.innerHTML = title;
	el.appendChild( elName );
	
	var elCmds = this._doc.createElement("DIV");
		elCmds.className = "commands";
		elCmds.setAttribute("gtdreviewid", id);
		if( !inbuilt ){
			var html = [];
			html.push("<span class='move-holder'>[");
			if( !firstInCategory ) html.push( "<span class='move-up'>" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.MoveUp") + "</span><span class='move-down-holder'>, " );
			if( firstInCategory ) html.push("<span class='move-down-holder'>");
			html.push( "<span class='move-down'>" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.MoveDown") + "</span></span>] </span>" );
			html.push( this._gtdinboxPage.stringBundle.getString("ModifyReviews.Edit") );
			html.push( " <a href='#' class='edit-title'>" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.EditTitle") + "</a>" );
			html.push( ", <a href='#' class='edit-category'>" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.EditCategory") + "</a>" );
			html.push( ", <a href='#' class='edit-search'>" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.EditSearch") + "</a>" );
			html.push( " <button class='edit-delete'>" + this._gtdinboxPage.stringBundle.getString("ModifyReviews.Delete") + "</button>" );
			elCmds.innerHTML = html.join("");
		}
	el.appendChild( elCmds );
		
	var elSearch = this._doc.createElement("DIV");
		elSearch.className = "search-query";
		elSearch.innerHTML = search;
	el.appendChild( elSearch );
	
	if( insertBefore ){
		this.elModifyReviews.insertBefore( el, insertBefore );
	}else{
		this.elModifyReviews.appendChild(el);
	}


	return el;
}catch(e){
	if( GTDDEBUGMODE ) alert("ModifyReviews _addItem\n" + e.toString());
}
};


ModifyReviews.prototype._click = function(event){
	// Detect what was clicked
	// Act appropriately
try{
	
	
	var reviews = this._gtdinboxPage.settings.reviews;
	
	var el = GIPROTOTYPE.element(event);
	
	if( el.parentNode.parentNode.hasAttribute("category") ){	// Clicked title based element
		var category = el.parentNode.parentNode.getAttribute("category");
		switch( el.className ){
			case "title-move-up":
				// Get previous category
				// Get first review in previous category
				// Move everything in this category to before that first review
				var trip = false;
				var extremeReview = null;
				var nextCategory = "";
				var reviewMarkerIndex = 0;
				for( var i = reviews.length - 1; i >= 0; i-- ){
					var reviewCategory = this._getCategory(reviews[i]);
					if( reviewCategory==category ) trip = true;
					if( trip==true && category!=reviewCategory ){	// Found previous one
						nextCategory = reviewCategory;
					}
					if( trip==true && nextCategory && nextCategory==this._getCategory(reviews[i]) ){	// Found review in next category
						extremeReview = reviews[i];
						reviewMarkerIndex = i;
					}
				}
				if( extremeReview ){	
					var tmpReviews = [];				
					for( var j = reviews.length - 1; j > 0; j-- ){
						if( this._getCategory(reviews[j])==category ){
							tmpReviews.push(reviews[j]);
							reviews.splice(j, 1);
						}
					}
					for( var k = 0; k < reviews.length; k++ ){
						if( reviews[k]==extremeReview ){
							reviewMarkerIndex = k - 1;
							break;
						}
					}
					for( var j = tmpReviews.length - 1; j >= 0;  j-- ){
						reviews.splice(reviewMarkerIndex, 0, tmpReviews[j]);
					}
				}
				break;
			case "title-move-down":
				// Get next category
				// Get first review in next category
				// Move everything in this category to after that first review
				var trip = false;
				var nextCategory = "";
				var reviewMarkerIndex = 0;
				for( var i = 0; i < reviews.length; i++ ){
					var reviewCategory = this._getCategory(reviews[i]);
					if( reviewCategory==category ) trip = true;
					if( trip==true && category!=reviewCategory ){	// Found next one
						nextCategory = reviewCategory;

						var markerReview = reviews[i];
						var tmpReviews = [];
						for( var j = reviews.length - 1; j >= 0; j-- ){
							if( this._getCategory(reviews[j])==category ){
								tmpReviews.push(reviews[j]);
								reviews.splice(j, 1);
							}
						}
						for( var k = 0; k < reviews.length; k++ ){
							if( reviews[k]==markerReview ){
								reviewMarkerIndex = k + 1;
								break;
							}
						}
						for( var j = 0; j < tmpReviews.length; j++ ){
							reviews.splice(reviewMarkerIndex, 0, tmpReviews[j]);
						}
						break;
					}
				}
				break;
		}
		this._gtdinboxPage.saveSettings();
		this.refresh();
		this._gtdinboxPage._dashboard.refresh();
		return;
	}
	
	var reviewId = "";
	var elParent = el.parentNode;
	while( elParent.parentNode ){
		reviewId = elParent.getAttribute("gtdreviewid");
		if( reviewId ) break;
		elParent = elParent.parentNode;
	}
	if( !reviewId || !el.className ) return;
		
	var review = null;
	var reviewIndex = 0;
	for( var i = 0; i < reviews.length; i++ ){
		if( reviews[i].id==reviewId ){
			review = reviews[i];
			reviewIndex = i;
			break;
		}
	}
	if( !review ) return;
	
	
	switch(el.className){
		case "edit-title":
			var title = prompt( this._gtdinboxPage.stringBundle.getString("ModifyReviews.PromptTitle"), review.title );
			if( title ) review.title = title;
			break;
		case "edit-category":
			var category = prompt( this._gtdinboxPage.stringBundle.getString("ModifyReviews.PromptCategory"), review.category );
			if( category ) review.category = category;
			break;
		case "edit-search":
			var search = prompt( this._gtdinboxPage.stringBundle.getString("ModifyReviews.PromptSearch"), review.search );
			if( search ) review.search = search;
			break;
		case "edit-delete":
			for( var i = 0; i < reviews.length; i++ ){
				if( reviews[i].id==reviewId ){
					reviews.splice(i,1);
					break;
				}
			}
			break;
		case "move-up":
			if( reviewIndex > 0 ){
				reviews.splice(reviewIndex, 1);	// remove old review
				reviews.splice(reviewIndex-1, 0, review); // insert in position before
			}
			break;
		case "move-down":
			if( reviewIndex < reviews.length -1 ){
				reviews.splice(reviewIndex, 1);
				reviews.splice(reviewIndex+1, 0, review);
			}
			break;
	}
	
	
	this._gtdinboxPage.saveSettings();
	this.refresh();
	this._gtdinboxPage._dashboard.refresh();
	
}catch(e){
	if( GTDDEBUGMODE ) alert("ModifyReviews _click\n" + e.toString());
}
};


ModifyReviews.prototype._clickCreate = function(event){
	// Create new review:
	var review = {};
	review.id = (new Date()).valueOf();
	review.title = this.elCreateTitle.value;
	review.category = this.elCreateCategory.value;
	review.search = this.elCreateSearch.value;

	// Check the input
	if( review.title=="" || review.search=="" ){
		alert( this._gtdinboxPage.stringBundle.getString("ModifyReviews.Create.NoValues") );
		return;
	}
	
	if( !this._gtdinboxPage.settings.reviews ) this._gtdinboxPage.settings.reviews = [];
	this._gtdinboxPage.settings.reviews.push( review );
	this._gtdinboxPage.saveSettings();
	
	this.refresh();
	this._gtdinboxPage._dashboard.refresh();
};
