function SearchBoxUI(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(this);	
	
	
	// Create it
	this.elInput = this._gtdinboxPage.gmailDOM.element("search-box");
	this.elSubmit = this._gtdinboxPage.gmailDOM.element("search-button");
	
	
	if( !this._gtdinboxPage.settings.ui.hide.searchBox ){
		this.elMenu = this._doc.createElement("SELECT");
			this.elMenu.className = "gtd-search-dropdown";
		this.elSubmit.parentNode.insertBefore( this.elMenu, this.elSubmit);
		this.refreshMenu();
		
		this._eventObservers.observe(this.elInput, "keydown", this._inputKey, true);
		this._eventObservers.observe(this.elSubmit, "keydown", this._submitKey, true);
		this._eventObservers.observe(this.elSubmit, "mouseup", this._submitClick, true);
		this._eventObservers.observe(this.elMenu, "keydown", this._menuKey, true);
	}
}

SearchBoxUI.prototype.refreshMenu = function(){
	if( this.elMenu ){
		this.elMenu.innerHTML = "";
		
		var list = GIPROTOTYPE.clone(this._gtdinboxPage.settings.searchBox.list);
		list.splice(0,0,"All,");
		for( var i = 0; i < list.length; i++ ){
			var searchData = list[i].split(",");
			var elOption = this._doc.createElement("OPTION");
				elOption.value = list[i];
				elOption.innerHTML = this._gtdinboxPage.stringBundle.getStringSafe("SearchBox.Menu." + searchData[0], searchData[0]);
			this.elMenu.appendChild( elOption );
		}
	}
};

// Override Search:
SearchBoxUI.prototype._inputKey = function(event){	// If Enter, change the input box
	if( event.keyCode==13 ) this._submitClick();
};
SearchBoxUI.prototype._submitKey = function(event){	// If Enter, change the input box
	if( event.keyCode==13 ) this._submitClick();
};
SearchBoxUI.prototype._menuKey = function(event){	// If Enter, change the input box
	if( event.keyCode==13 ){
		this._submitClick();
		GIPROTOTYPE.doMouseEvent( this._doc, this.elSubmit, "click");
	}
};
SearchBoxUI.prototype._submitClick = function(event){ // Change the input box
try{
	var searchDataStr = this.elMenu.value;
	var searchData = searchDataStr.split(",");
	var search = this._gtdinboxPage.replaceGtdVariables(searchData[1]);	// From, from:, prefix
	
	if( searchData[2] && searchData[2].toLowerCase()=="prefix" ){
		if( this.elInput.value.indexOf(search)!=0 ) this.elInput.value = search + this.elInput.value.replace(/\/|\s|\(|\)/g, "-");
	}else{
		if( this.elInput.value.indexOf(search)==-1 ) this.elInput.value += " " + search;
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("SearchBoxUI _submitClick\n" + e.toString());
}
};

// Misc Search:
SearchBoxUI.prototype.search = function(str){
	this.elMenu.selectedIndex = 0;
	this.elInput.value = str;
	GIPROTOTYPE.doMouseEvent( this._doc, this.elSubmit, "click");
};