function SettingsGmail(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(this);	
	this._eventObserversHeader = new GtdEventObservers(this);	
	this._labels = gtdinboxPage._labels;
	this._sp = gtdinboxPage.sp;
	this._multiToOne = new MultiToOne(this._gtdinboxPage);
	
	this.elAlertBox = this._gtdinboxPage.gmailDOM.element("alert-box");
	this._eventObservers.observe( this.elAlertBox, "DOMNodeInserted", this.alertBoxChange, false );
	
};


SettingsGmail.prototype.alertBoxChange = function(event){
	this._multiToOne.process("alert-box-change", GIPROTOTYPE.bind(this, this._alertBoxChange));
};
SettingsGmail.prototype._alertBoxChange = function(event){
try{
	if( this._gtdinboxPage.view()=="s" ){
		this.viewChange();
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("SettingsGmail alertBoxChange\n" + e.toString());
}
};

SettingsGmail.prototype.viewChange = function(){
	
	var elHeader = this._gtdinboxPage.gmailDOM.element("settings-header");
	
	// Add the GTDInbox tab
	var found = false;
	for( var i = 0; i < elHeader.childNodes.length; i++ ){
		if( elHeader.childNodes[i].id=="gtdinbox-settings" ){
			found = true;
			break;
		}
	}
	if( !found ){
		this.elTab = this._doc.createElement("DIV");
			this.elTab.id = "gtdinbox-settings";
			this.elTab.className = "AiCerc";
			this.elTab.innerHTML = "<a class='NAlyed' href='#'>GTDInbox</a>";
		elHeader.insertBefore( this.elTab, elHeader.lastChild );
	}
		
	// Listen for clicks on tabs:
	this._eventObserversHeader.reset();
	this._eventObserversHeader.observe( elHeader, "mouseup", this.clickTabs, true );
	
	// Update the current view
	this.clickTabs();
};

SettingsGmail.prototype._reloadOptionsUI = function(){
	var elContainer = this._gtdinboxPage.gmailDOM.element("settings-content-container");
	
	// Remove any existing instances:
	for( var i = 0; i < elContainer.childNodes.length; i++ ){
		if( elContainer.childNodes[i].id=="gtdinbox-settings-options" ){
			elContainer.removeChild( elContainer.childNodes[i] );
			i--;
		}
	}
	
	// Create the new content window for us:
	this.elOptions = this._doc.createElement("DIV");
		this.elOptions.className = "XoqCub";
		this.elOptions.id = "gtdinbox-settings-options";
	this.elOptions.innerHTML = "<div class='XoqCub eGbrz'><table width='100%'><tbody><tr><td></td></tr></tbody></table></div>";	// <table width='100%'><tbody></tbody></table>
	elContainer.appendChild( this.elOptions );
	var elOptionsContent = this.elOptions.getElementsByTagName("TD")[0];
	
	var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile("options") );
	var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
	elOptionsContent.innerHTML = match[1];
		
	this._options = new GtdOptions(this._gtdinboxPage, elOptionsContent, this);
		this._options.addEventHandler( this, "refresh-options", this._reloadOptionsUI );
	this._modifyReviews = new ModifyReviews(this._gtdinboxPage, elOptionsContent, this);
};

SettingsGmail.prototype.clickTabs = function(event){
try{
	
	var doc = this._gtdinboxPage.gmailDOM.element("document");
	
	if( event ){
		var el = GIPROTOTYPE.element(event);
		el = el.parentNode;	// from A to DIV
		if( el.id=="gtdinbox-settings" && el.className!="ZOB6Zc" ){	// clicked gtdinbox settings
			
			this._lastTab = {};
			
			// Move tabs:
			var elHeader = this._gtdinboxPage.gmailDOM.element("settings-header");
			for( var i = 0; i < elHeader.childNodes.length; i++ ){
				if( elHeader.childNodes[i].className=="ZOB6Zc" ){
					this._lastTab.tab = elHeader.childNodes[i];
					elHeader.childNodes[i].className = "AiCerc"; // remove highlight
					break;
				}
			}
			this.elTab.className = "ZOB6Zc";
			
			var elContainer = this._gtdinboxPage.gmailDOM.element("settings-content-container");
			for( var i = 0; i < elContainer.childNodes.length; i++ ){
				if( elContainer.childNodes[i].style.display!="none" ){
					this._lastTab.content = elContainer.childNodes[i];
					elContainer.childNodes[i].style.display = "none"; // hide other content windows
					break;
				}
			}
			
			this._reloadOptionsUI();
		}else{
			// Reset it
			this.elTab.className = "AiCerc";
			this.elOptions.style.display = "none";
			if( this._lastTab && this._lastTab.tab ){
				this._lastTab.tab.className = "ZOB6Zc";
				this._lastTab.content.style.display = "";
				this._lastTab = null;
			}
		}
	}else{
		this.elTab.className = "AiCerc";
	}
	
	
	if( this._gtdinboxPage.gmailDOM.element("labels-create-box") ){	// It's the labels screen. Hook in.
		
		if( !this.elPrefix || !doc.getElementById("gtdinbox.labels-prefix") ){
			this.elPrefix = this._doc.createElement("SELECT");
				this.elPrefix.id = "gtdinbox.labels-prefix";
				this.elPrefix.className = "labels-prefix";
			
			var elOption = this._doc.createElement("OPTION");
				elOption.innerHTML = "";
			this.elPrefix.appendChild( elOption );
			
			var types = this._gtdinboxPage.settings.labels.types;
			for( t in types ){
				if( types[t].prefix!="" ){
					var elOption = this._doc.createElement("OPTION");
					elOption.innerHTML = types[t].prefix;
					elOption.value = types[t].prefix;
					this.elPrefix.appendChild( elOption );
				}
			}
			
			this.elInput = this._gtdinboxPage.gmailDOM.element("labels-create-box");
			if( this.elInput ){
				this.elInput.parentNode.insertBefore(this.elPrefix, this.elInput);
			
				var elCreate = this.elInput.parentNode.getElementsByTagName("BUTTON")[0];
				this._eventObservers.observe( elCreate, "mousedown", this._clickCreate, true);
			}
		}
		
		
		if( !this.elInstall || !doc.getElementById("gtdinbox.install") ){
			this.elInstall = this._doc.createElement("DIV");
				this.elInstall.id = "gtdinbox.install";
				this.elInstall.className = "labels-install";
				this.elInstall.style.display = "none";
			
			this._gtdinboxPage.gmailDOM.element("labels-last-row").appendChild( this.elInstall );
			
			var elButton = this._doc.createElement("BUTTON");
				elButton.innerHTML = this._gtdinboxPage.stringBundle.getString("SettingsGmail.Install");
			this.elInstall.appendChild( elButton );
			
			this._eventObservers.observe( elButton, "click", this._clickInstall, false );
		}

		if( !this.elHelp || !doc.getElementById("gtdinbox.labels-help") ){
			this.elHelp = this._doc.createElement("DIV");
				this.elHelp.id = "gtdinbox.labels-help";
				this.elHelp.className = "labels-help";
			
			this._gtdinboxPage.gmailDOM.element("labels-last-row").appendChild( this.elHelp );

			var el = this._doc.createElement("DIV");
				var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile("labels") );
				var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
				el.innerHTML = match[1];
			this.elHelp.appendChild( el );	
		}
		
		
		this.testBasicLabels();
	}

}catch(e){
	if( GTDDEBUGMODE ) alert( "SettingsGmail clickTabs\n" + e.toString() );
}
};

SettingsGmail.prototype._clickCreate = function(event){
	if( this.elInput ) this.elInput.value = this.elPrefix.value + this.elInput.value;
	this.testBasicLabels();
};

SettingsGmail.prototype._clickInstall = function(event){
try{
	this._gtdinboxPage._lightbox.setType("setup-and-close");
	this._gtdinboxPage._lightbox.active(true);
}catch(e){
	if( GTDDEBUGMODE ) alert( "SettingsGmail _clickInstall\n" + e.toString() );
}
};

SettingsGmail.prototype.testBasicLabels = function(){
	// If basic labels, hide install; else show it
try{	
	var labelsHash = this._gtdinboxPage._labels.getLabelsHash();
	
	var settings = this._gtdinboxPage.settings;
	
	
	var t = "Statuses";
	var prefix = settings.labels.types[t].prefix;
	for( d in settings.labels.types[t].defaults ){
		var label = prefix + settings.labels.types[t].defaults[d];
		if( labelsHash[label] ){
			this.elInstall.style.display = "none";
			return true;
		}
	}
	
	// No labels found:
	this.elInstall.style.display = "";
	return false;
}catch(e){
	if( GTDDEBUGMODE ) alert( "SettingsGmail testBasicLabels\n" + e.toString() );
}
};

SettingsGmail.prototype.destroy = function(){
	this._eventObservers.destroy();
	this._eventObserversHeader.destroy();
	this._multiToOne.destroy();

	this._gtdinboxPage = null;
	this._doc = null;
};