/* Copyright Andy Mitchell 2007. No warranties */

/*
Syncing to Server
Because we use auto-generated IDs, if two copies of GTDInbox are running at once; it is feasible the same label (or thread) can have two different IDs.
So, when sync'd with the server (either initial download, or in response to upload); make sure we update all label IDs against the server.
*/

function StoredProceduresGTD(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._db = gtdinboxPage.db;

}

/* INSERTION */

StoredProceduresGTD.prototype._safeInput = function(str, borderQuotes){
	str = str.replace(/\"/g, "\\\"")
	if( borderQuotes ) str = "\"" + str + "\"";
	return str;
};
StoredProceduresGTD.prototype._getAccount = function(){
	if( !this._account ) this._account = this._gtdinboxPage._environment.email;
	return this._account;
};


StoredProceduresGTD.prototype._insert = function(table, fields, values){	
	fields += ", account, date";
	values += ", \"" + this._getAccount() + "\", " + (new Date()).valueOf();
	this._db.execute("INSERT INTO " + table + " (" + fields + ") VALUES(" + values + ")");
};

/*
StoredProceduresGTD.prototype.addThread = function(thread){
	this._insert( "threads", "thread", this._safeInput(thread, true) );
};

StoredProceduresGTD.prototype.addLabel = function(label){
	this._insert( "labels", "label", this._safeInput(label, true) );
};

StoredProceduresGTD.prototype.addContact = function(email){
	this._insert( "contacts", "contact", this._safeInput(email, true) );
};
*/

StoredProceduresGTD.prototype.addAutoLabelled = function(msg, label){
	this._insert("auto_labelled", "label, msgid", this._safeInput(label, true) + ", " + this._safeInput(msg, true));
};

StoredProceduresGTD.prototype.addThreadContacts = function(thread, senders, recipients){
	//this.addThread(thread);
	this._addThreadContacts(thread, senders, 0);
	this._addThreadContacts(thread, recipients, 1);
};
StoredProceduresGTD.prototype._addThreadContacts = function(thread, contacts, type){
	if( !contacts ) return;
	if( !(contacts instanceof Array ) ) contacts = [contacts];
	for( var i = 0; i < contacts.length; i++ ){
		//this.addContact(contacts[i]);
		contacts[i][1] = contacts[i][1].toLowerCase();
		this._insert("thread_contacts", "thread, contact, type", this._safeInput(thread, true) + ", " + this._safeInput(contacts[i][1],true) + ", " + type);
		this._insert("contact_emails", "contact_name, contact_email", this._safeInput(contacts[i][0], true) + ", " + this._safeInput(contacts[i][1], true) );
	}	
};

StoredProceduresGTD.prototype.addRecentNewsItem = function(url){
	this._insert("recent_news_items", "url", this._safeInput(url, true));
};

StoredProceduresGTD.prototype.addThreadLabel = function(thread, label){
	//this.addThread(thread);
	//this.addLabel(label);
	
	// Check if it exists
	
	var sql = "SELECT DISTINCT label FROM thread_labels WHERE label=\"" + label + "\" AND account = \"" + this._getAccount() + "\"";
	var result = {found:false};
	var fRow = function(statement){
			result.found = true;
		};
	this._db.select(sql, fRow)
	
	if( result.found ){	// Update last date
		this._db.execute("UPDATE thread_labels SET date = " + (new Date()).valueOf() + " WHERE label=\"" + label + "\" AND account = \"" + this._getAccount() + "\"");
	}else{	// Add
		this._insert( "thread_labels", "thread, label", this._safeInput(thread, true) + ", " + this._safeInput(label, true) );
	}
};


/* RETRIEVAL */

/*
StoredProceduresGTD.prototype._getLabelThreadSql = function(){
	return "SELECT label, thread_id, thread_labels.date AS thread_labels_date, labels.date AS labels_date FROM labels INNER JOIN thread_labels ON labels.id = thread_labels.label_id WHERE (((labels.account)=\"" + this._getAccount() + "\"))"
};
StoredProceduresGTD.prototype._getContactThreadSql = function(){
	return "SELECT DISTINCT contact, thread_id, thread_contacts.date AS thread_contacts_date, contacts.date AS contacts_date FROM contacts INNER JOIN thread_contacts ON contacts.id = thread_contacts.contact_id WHERE (((contacts.account)=\"" + this._getAccount() + "\"))"
};
*/

/*
StoredProceduresGTD.prototype.getThreadId = function(thread){
	var result = {};
	var fRow = function(statement){
			result.id = statement.getString(0);
		};
	this._db.select("SELECT id FROM threads WHERE thread = \"" + this._safeInput(thread) + "\" AND account = \"" + this._getAccount() + "\"", fRow);
	return result.id || -1;
};
StoredProceduresGTD.prototype.getLabelId = function(label){
	var result = {};
	var fRow = function(statement){
			result.id = statement.getString(0);
		};
	this._db.select("SELECT id FROM labels WHERE label = \"" + this._safeInput(label) + "\" AND account = \"" + this._getAccount() + "\"", fRow);
	return result.id || -1;
};
StoredProceduresGTD.prototype.getContactId = function(contact){
	var result = {};
	var fRow = function(statement){
			result.id = statement.getString(0);
		};
	this._db.select("SELECT id FROM contacts WHERE contact = \"" + this._safeInput(contact) + "\" AND account = \"" + this._getAccount() + "\"", fRow);
	return result.id || -1;
};
*/

StoredProceduresGTD.prototype.getRecentNewsItems = function(){
	var sql = "SELECT DISTINCT url FROM recent_news_items WHERE account = \"" + this._getAccount() + "\" ORDER BY date DESC LIMIT 30";
	
	var results = {};
	var fRow = function(statement){
			results[statement.getString(0)] = true;
		};
	this._db.select(sql, fRow)
	return results;
};

StoredProceduresGTD.prototype.getRecentlyAttachedProjects = function(){
	var projectPrefix = this._gtdinboxPage.settings.labels.types.Projects.prefix;
	
	var sql = "SELECT DISTINCT label FROM thread_labels WHERE label Like \"" + projectPrefix + "%\" AND account = \"" + this._getAccount() + "\" ORDER BY date DESC"
	
	var results = [];
	var fRow = function(statement){
			results.push(statement.getString(0));
		};
	this._db.select(sql, fRow)
	return results;
};

StoredProceduresGTD.prototype.getEmailForContactName = function(name){
	var sql = "SELECT contact_email FROM contact_emails WHERE contact_name = " + this._safeInput(name, true);
	var result = {email:""};
	var fRow = function(statement){
			result.email = statement.getString(0);
		};
	this._db.select(sql, fRow)
	return result.email;
};

StoredProceduresGTD.prototype.getRecentProjectsForContacts = function(contacts, useHash){
	var projectPrefix = this._gtdinboxPage.settings.labels.types.Projects.prefix;
	if( typeof contacts=="string" ) contacts = [contacts];

	var sql = "SELECT label"
	sql += " FROM thread_contacts INNER JOIN thread_labels ON thread_contacts.thread = thread_labels.thread";
	sql += " WHERE thread_labels.label Like \"" + projectPrefix + "%\" AND thread_labels.account = \"" + this._getAccount() + "\" AND (";
	for( var i = 0; i < contacts.length; i++ ){
		if( i>0 ) sql += " OR ";
		sql += "(contact=\"" + contacts[i] + "\")";
	}
	sql += ") ORDER BY thread_labels.date DESC";

	var results = useHash? {} : [];
	var fRow = function(statement){
			if( useHash ){
				results[statement.getString(0)] = true;
			}else{
				results.push(statement.getString(0));
			}
		};
	this._db.select(sql, fRow)
	return results;
};

StoredProceduresGTD.prototype.getRankedContactsAssociatedWithProject = function(project){
	var sql = "SELECT contact, Count(contact) AS CountOfContact";
	sql += " FROM thread_contacts INNER JOIN thread_labels ON thread_contacts.thread = thread_labels.thread";
	sql += " GROUP BY contact, label";
	sql += " HAVING thread_labels.label=\"" + project + "\" AND thread_contacts.account=\"" + this._getAccount() + "\" AND thread_labels.account=\"" + this._getAccount() + "\"";
	sql += " ORDER BY Count(thread_contacts.contact) DESC";
	
	var results = [];
	var fRow = function(statement){
			results.push(statement.getString(0));
		};
	this._db.select(sql, fRow)
	return results;
};

StoredProceduresGTD.prototype.getRankedProjectsAssociatedWithContact = function(contacts, useHash){
	var projectPrefix = this._gtdinboxPage.settings.labels.types.Projects.prefix;
	if( typeof contacts=="string" ) contacts = [contacts];
		
	var sql = "SELECT label, Count(thread_labels.label) AS CountOflabel";
	sql += " FROM thread_contacts INNER JOIN thread_labels ON thread_contacts.thread = thread_labels.thread";
	sql += " GROUP BY thread_labels.label, thread_labels.account, thread_contacts.account, thread_contacts.contact";
	sql += " HAVING label Like \"" + projectPrefix + "%\" AND thread_labels.account=\"" + this._getAccount() + "\" AND thread_contacts.account=\"" + this._getAccount() + "\" AND (";
	for( var i = 0; i < contacts.length; i++ ){
		if( i>0 ) sql += " OR ";
		sql += "(contact=\"" + contacts[i] + "\")";
	}
	sql += " ) ORDER BY Count(thread_labels.label) DESC";

	var results = useHash? {} : [];
	var fRow = function(statement){
			if( useHash ){
				results[statement.getString(0)] = true;
			}else{
				results.push(statement.getString(0));
			}
		};
	this._db.select(sql, fRow)
	return results;
};

StoredProceduresGTD.prototype.getRankedLabelsByLabel = function(prefix, labels){
	if( typeof labels=="string" ) labels = [labels];
				
	var sql = "SELECT thread_labels_1.label, Count(thread_labels_1.label) AS CountOflabel, thread_labels.account ";
	sql += "FROM thread_labels AS thread_labels_1 INNER JOIN thread_labels ON thread_labels_1.thread = thread_labels.thread ";
	sql += "GROUP BY thread_labels_1.label, thread_labels.account, thread_labels.label ";
	sql += "HAVING (";
	for( var i = 0; i < labels.length; i++ ){
		if( i>0 ) sql += "OR ";
		sql += "(NOT thread_labels_1.label=\"" + labels[i] + "\") ";
	}
	sql += ") AND (thread_labels.account=\"" + this._getAccount() + "\") AND (thread_labels_1.account=\"" + this._getAccount() + "\") ";
	sql += "AND thread_labels_1.label LIKE \"" + prefix + "%\" AND (";
	for( var i = 0; i < labels.length; i++ ){
		if( i>0 ) sql += "OR ";
		sql += "(thread_labels.label=\"" + labels[i] + "\") ";
	}
	sql += ") ORDER BY Count(thread_labels_1.label) DESC";

	var results = [];
	var fRow = function(statement){
			results.push(statement.getString(0));
		};
	this._db.select(sql, fRow)
	return results;
};

StoredProceduresGTD.prototype.getRankedContactsByLabel = function(labels){
	if( typeof labels=="string" ) labels = [labels];

	var sql = "SELECT thread_contacts.contact, Count(thread_contacts.contact) AS CountOfcontact, thread_contacts.account, thread_labels.account ";
	sql += "FROM thread_contacts INNER JOIN thread_labels ON thread_contacts.thread = thread_labels.thread ";
	sql += "GROUP BY thread_contacts.contact, thread_contacts.account, thread_labels.account, thread_labels.label ";
	sql += "HAVING (thread_contacts.account=\"" + this._getAccount() + "\") ";
	sql += "AND (thread_labels.account=\"" + this._getAccount() + "\") AND (";
	for( var i = 0; i < labels.length; i++ ){
		if( i>0 ) sql += "OR ";
		sql += "(thread_labels.label=\"" + labels[i] + "\") ";
	}
	sql += ") ORDER BY Count(thread_contacts.contact) DESC";

	var results = [];
	var fRow = function(statement){
			results.push(statement.getString(0));
		};
	this._db.select(sql, fRow);
	return results;
};

StoredProceduresGTD.prototype.getRankedLabelsByContact = function(prefix, contacts, useHash){
	if( typeof contacts=="string" ) contacts = [contacts];

	var sql = "SELECT thread_labels.label, Count(thread_labels.label) AS CountOflabel, thread_labels.account, thread_contacts.account, thread_contacts.contact ";
	sql += "FROM thread_contacts INNER JOIN thread_labels ON thread_contacts.thread = thread_labels.thread ";
	sql += "GROUP BY thread_labels.label, thread_labels.account, thread_contacts.account, thread_contacts.contact ";
	sql += "HAVING (thread_labels.label Like \"" + prefix + "%\") ";
	sql += "AND (thread_labels.account=\"" + this._getAccount() + "\") ";
	sql += "AND (thread_contacts.account=\"" + this._getAccount() + "\") ";
	sql += "AND ( ";
	for( var i = 0; i < contacts.length; i++ ){
		if( i>0 ) sql += "OR ";
		sql += "(thread_contacts.contact=\"" + contacts[i] + "\") ";
	}
	sql += ") ORDER BY Count(thread_labels.label) DESC ";

	var results = useHash? {} : [];
	var fRow = function(statement){
			if( useHash ){
				results[statement.getString(0)] = true;
			}else{
				results.push(statement.getString(0));
			}
		};
	this._db.select(sql, fRow);
	return results;
};

StoredProceduresGTD.prototype.getRankedContactsByContact = function(contacts, useHash){
	if( typeof contacts=="string" ) contacts = [contacts];
	var contactsHash = GIPROTOTYPE.arrayToHash(contacts, true);

	var sql = "SELECT thread_contacts_1.contact, Count(thread_contacts_1.contact) AS CountOfcontact ";
	sql += "FROM thread_contacts INNER JOIN thread_contacts AS thread_contacts_1 ON thread_contacts.thread = thread_contacts_1.thread ";
	sql += "GROUP BY thread_contacts.account, thread_contacts.contact, thread_contacts_1.contact ";
	sql += "HAVING (thread_contacts.account=\"" + this._getAccount() + "\") ";
	sql += "AND ( ";
	for( var i = 0; i < contacts.length; i++ ){
		if( i>0 ) sql += "OR ";
		sql += "(thread_contacts.contact=\"" + contacts[i] + "\") ";
	}
	sql += ") ORDER BY Count(thread_contacts_1.contact) DESC";

	var results = useHash? {} : [];
	var fRow = function(statement){
			var email = statement.getString(0).toLowerCase();;
			if( !contactsHash[email] ){	// originals not in
				if( useHash ){
					results[email] = true;
				}else{
					results.push(email);
				}
			}
		};
	this._db.select(sql, fRow);
	return results;
};

StoredProceduresGTD.prototype.getAutoLabelled  = function(msgs){
	if( typeof msgs=="string" ) msgs = [msgs];
	var sql = "SELECT DISTINCT label, msgid FROM auto_labelled WHERE (";
	for( var i = 0; i < msgs.length; i++ ){
		if( i>0 ) sql += " OR ";
		sql += "msgid = \"" + msgs[i] + "\"";
	}
	sql += ") AND account = \"" + this._getAccount() + "\"";

	var results = [];
	var fRow = function(statement){
			results.push( [statement.getString(0), statement.getString(1)] );
		};
	this._db.select(sql, fRow, true);
	return results;
};

StoredProceduresGTD.prototype.countLabels  = function(label){
	var sql = "SELECT count(label) as countLabel FROM thread_labels WHERE label = " + this._safeInput(label,true);

	var result = {count:0};
	var fRow = function(statement){
			result.count = statement.getInt64(0);
		};
	this._db.select(sql, fRow);
	return result.count;
};


/* UPDATE */

StoredProceduresGTD.prototype.updateLabelName = function(oldname, newname){
	this._db.execute("UPDATE thread_labels SET label = \"" + this._safeInput(newname) + "\", session_modified = NULL WHERE label = \"" + this._safeInput(oldname) + "\" AND account = \"" + this._getAccount() + "\"");
	this._db.execute("UPDATE auto_labelled SET label = \"" + this._safeInput(newname) + "\", session_modified = NULL WHERE label = \"" + this._safeInput(oldname) + "\" AND account = \"" + this._getAccount() + "\"");
};

/* DELETION */

StoredProceduresGTD.prototype.removeThreadLabel = function(thread, label){
	this._db.execute("DELETE FROM thread_labels WHERE thread = " + this._safeInput(thread, true) + " AND label = " + this._safeInput(label, true) + " AND account = \"" + this._getAccount() + "\"");
};