function TaskPaneUI(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._groups = {};
	this._eventObservers = new GtdEventObservers(this);
	
	try{
		this.el = this._doc.createElement("DIV");
			this.el.className = "taskpane";
			
		this._eventObservers.observe(this.el, "click", this._click, false);
		
		// Create scoping:
		var elScopeBox = this._doc.createElement("DIV");
			elScopeBox.className = "scope";
			elScopeBox.innerHTML = this._gtdinboxPage.stringBundle.getStringSafe("TaskPane.Scope") + ": ";
		this.elScope = this._doc.createElement("SELECT");
			var elOption = this._doc.createElement("OPTION");
				elOption.innerHTML = this._gtdinboxPage.stringBundle.getStringSafe("TaskPane.Scope.CurrentView");
				elOption.value = "currentview";
			this.elScope.appendChild( elOption );
			var elOption = this._doc.createElement("OPTION");
				elOption.innerHTML = this._gtdinboxPage.stringBundle.getStringSafe("TaskPane.Scope.AllMail");
				elOption.value = "allmail";
			this.elScope.appendChild( elOption );
		elScopeBox.appendChild( this.elScope );
		this.el.appendChild( elScopeBox );
		this._eventObservers.observe( this.elScope, "change", this._changeScope, false );
		
		var status = this._gtdinboxPage.getStatusData(true);
		
		this._createGroup("TasksToDo");
			this._createGroupItem("TasksToDo", "NextAction", "ThreadList", [status.NextAction]);
			this._createGroupItem("TasksToDo", "Action", "ThreadList", [status.Action]);
			this._createGroupItem("TasksToDo", "SomeDay", "ThreadList", [status.SomeDay]);
			for( var i = 0; i < status.additional.length; i++ ){
				this._createGroupItem("TasksToDo", status.additional[i], "ThreadList", status.additional[i]);
			}
			var statusLabels = GIPROTOTYPE.clone(this._gtdinboxPage._labels.getLabelsArray("Statuses"));
			for( var i = statusLabels.length; i>=0; i-- ) if( statusLabels[i]==status.Finished || statusLabels[i]==status.WaitingOn ) statusLabels.splice(i,1);
			this._createGroupItem("TasksToDo", "Contact", "Contact", statusLabels);
			this._createGroupItem("TasksToDo", "Project", "Project", statusLabels);
		
		this._createGroup("TasksWaitingOn");
			this._createGroupItem("TasksWaitingOn", "WaitingOn", "ThreadList", [status.WaitingOn]);
			this._createGroupItem("TasksWaitingOn", "Contact", "Contact", [status.WaitingOn]);
			this._createGroupItem("TasksWaitingOn", "Project", "Project", [status.WaitingOn]);
		
		
		// Add Reviews:
		var elTitle = this._doc.createElement("DIV");
			elTitle.innerHTML = this._gtdinboxPage.stringBundle.getStringSafe("TaskPane.Reviews");;
			elTitle.className = "group-title";
		this.el.appendChild( elTitle );
		var elReviewUl = null;
		var elReviews = this._doc.createElement("DIV");
			elReviews.className = "reviews";
		this.el.appendChild( elReviews );
			var reviews = this._gtdinboxPage.settings.reviews;
			if( reviews ){
				// Organise
				var categories = {};
				for( var i = 0; i < reviews.length; i++ ){
					var title = reviews[i].title;
					if( !title && reviews[i].titleKey ) title = this._gtdinboxPage.stringBundle.getStringSafe(reviews[i].titleKey);
					
					var category = reviews[i].category;
					if( !category && reviews[i].categoryKey ) category = this._gtdinboxPage.stringBundle.getStringSafe(reviews[i].categoryKey);
					if( !category ) category = this._gtdinboxPage.stringBundle.getStringSafe("Dashboard.Reviews.Misc");
					
					var search = reviews[i].search;
					
					if( !categories[category] ) categories[category] = {reviews:[]};
					categories[category].reviews.push( {'title':title, 'search':search} );
				}
				// Render
				var seenCategories = {};
				for( c in categories ){
					for( var i = 0; i < categories[c].reviews.length; i++ ){
						if( !seenCategories[c] ){
							var elTitle = this._doc.createElement("DIV");
								elTitle.innerHTML = c;
								elTitle.className = "subgroup-title";
							elReviews.appendChild( elTitle );
							elReviewUl = this._doc.createElement("UL");
							elReviews.appendChild( elReviewUl );
							
							//reviewGroup.accordion.addGroupTitle(c);
							seenCategories[c] = true;
						}
						var elLi = this._doc.createElement("LI");
							elLi.innerHTML = categories[c].reviews[i].title;
							elLi.setAttribute("gtdSearch", categories[c].reviews[i].search);
						elReviewUl.appendChild( elLi );
						//this._createGroupItem( "Reviews", categories[c].reviews[i].title, "ThreadList", null, categories[c].reviews[i].search );
					}
				}
			}
		
	}catch(e){
		if( GTDDEBUGMODE ) alert("TaskPaneUI create\n" + e.toString());
	}
}

TaskPaneUI.prototype._createGroup = function(titleKey, overrideCSS){
try{
	var title = this._gtdinboxPage.stringBundle.getStringSafe("TaskPane." + titleKey, titleKey);
	//this._gtdinboxPage.unsafeWindow.console.log("_createGroup -> TaskPane." + titleKey + " => " + title);
	
	var elTitle = this._doc.createElement("DIV");
		elTitle.className = (overrideCSS || "group-title");
		elTitle.innerHTML = title;
	this.el.appendChild( elTitle );
	
	var accordion = new AccordionUI(this._gtdinboxPage, this.el);
	
	// Store it
	this._groups[titleKey] = {'titleKey':titleKey, 'elTitle':elTitle, 'accordion':accordion};
	return this._groups[titleKey];
}catch(e){
	if( GTDDEBUGMODE ) alert("TaskPaneUI _createGroup\n" + e.toString());
}
};

TaskPaneUI.prototype._createGroupItem = function(parentTitleKey, titleKey, type, labels, search){
try{
	var title = this._gtdinboxPage.stringBundle.getStringSafe("TaskPane." + parentTitleKey + "." + titleKey, titleKey);
	
	// Add it to the accordion:
	var section = this._groups[parentTitleKey].accordion.addSection(title);
	if( type=="ThreadList" ){
		section.threadListUI = new ThreadListCustomUI(this._gtdinboxPage);
		section.elContent.appendChild( section.threadListUI.el );
	}
	
	
	// Store it
	if( !this._groups[parentTitleKey].items ) this._groups[parentTitleKey].items = {};
	this._groups[parentTitleKey].items[titleKey] = {'titleKey':titleKey, 'title':title, 'accordion':this._groups[parentTitleKey].accordion, 'labels':labels, 'search':search, 'section':section, 'type':type};

}catch(e){
	if( GTDDEBUGMODE ) alert("TaskPaneUI _createGroupItem\n" + e.toString());
}
};

TaskPaneUI.prototype._changeScope = function(event){
	this.refresh();
};

TaskPaneUI.prototype.refresh = function(){
	// Discern the parent search (from scope)
	// Search for all items with tasks within the parent search. 
	// For all items:
	//	If ThreadList:
	//		If Search: Create a new thread list search for this item and render threadlist
	//		If Labels: Restrict the threads to match those threads; and render threadlist
	//	If Project: Extract and rank Project from the list
	//	If Contact: Extract and rank Contact from the list

try{	
	if( !this._searcher ) this._searcher = new Searcher(this._gtdinboxPage);
	
	var parentSearch = "";
	if( this.elScope.value=="currentview" ){
		parentSearch = this._gtdinboxPage.gmailDOM.extractSearchQuery() +  " ";
	}
	var status = this._gtdinboxPage.getStatusData(true);
	
	var ajaxCalls = {count:0};
	var fAllCallbacksDone = GIPROTOTYPE.bind( this, function(){
			// Select the first non-disabled accordion item of every type
			for( g in this._groups ){
				for( i in this._groups[g].items ){
					var item = this._groups[g].items[i];
					item.accordion.currentIsFirstNonDisabled();
				}
			}
		});
	
	// Load up all the tasks:
	var taskQuerySearch = parentSearch + status.orQueryAll;
	this._searcher.search( taskQuerySearch, GIPROTOTYPE.bind(this, function(threadList){
			this._refreshCallbackTasks(threadList);
			if( --ajaxCalls.count<= 0 ) fAllCallbacksDone();
		}));
	ajaxCalls.count++;
	
	// Clear out previous efforts, fire custom searches, and link up the accordion header
	var itemHash = {};
	for( g in this._groups ){
		for( i in this._groups[g].items ){
			var item = this._groups[g].items[i];
			if( item.section.threadListUI ) item.section.threadListUI.el.innerHTML = "";
			else item.section.elContent.innerHTML = "";
			
			if( item.labels ){
				var labelSearch = "";
				for( var j = 0; j < item.labels.length; j++ ) labelSearch += GIPROTOTYPE.searchableLabel(item.labels[j]) + " OR ";
				labelSearch = "(" + labelSearch.replace(/ OR $/, "") + ")";
				item.search = taskQuerySearch + " " + labelSearch;
				switch( item.type ){
					case "ThreadList":
						item.section.elTitleSpan.setAttribute("gtdSearch", item.search);
						GIPROTOTYPE.addClassName( item.section.elTitleSpan, "title-link" );
						break;
				}
			}else{
				var searchStr = parentSearch + this._gtdinboxPage.replaceGtdVariables(item.search);
				itemHash[searchStr] = item;
				this._searcher.search( searchStr, GIPROTOTYPE.bind( this, function(threadList, query){
						var item = itemHash[query];	// Render ThreadList
						this._refreshCallbackRender(item, threadList);
						if( --ajaxCalls.count<= 0 ) fAllCallbacksDone();
					}));
				ajaxCalls.count++;
				item.section.elTitleSpan.setAttribute("gtdSearch", searchStr);
				GIPROTOTYPE.addClassName( item.section.elTitleSpan, "title-link" );
			}
		}
	}
	

}catch(e){
	if( GTDDEBUGMODE ) alert("TaskPaneUI refresh\n" + e.toString());
}
	
};
TaskPaneUI.prototype._refreshCallbackTasks = function(threadList){
try{
	// Accept a single thread list containing all actions; and for each item, restrict to labels and 
	for( g in this._groups ){
		for( i in this._groups[g].items ){
			var item = this._groups[g].items[i];
			if( item.labels ){	// It wants restricting to this task list
				var tl = item.labels.length==0? threadList : this._searcher.restrictThreadListToLabels(threadList, item.labels);	// Cleanse the thread list
				this._refreshCallbackRender(item, tl);
			}
		}
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("TaskPaneUI _refreshCallbackTasks\n" + e.toString());
}
};
TaskPaneUI.prototype._refreshCallbackRender = function(item, threadList){
try{
	if( item.type=="ThreadList" ){
		// Just load the ThreadListCustomUI
		item.section.threadListUI.load( threadList, null, {'excludeNames':true, 'excludeCmds': true, 'excludeDate':true, 'excludeLabels':true, 'tooltipNames': true, 'tooltipDate': true});
		this._updateAccordionTitleCount(item, threadList._threads.length );
	}else if( item.type=="Project" || item.type=="Contact" ){
		var liCounter = {count:0};
		this._searcher.renderThreadListByType(item.type, threadList, "volume", null, GIPROTOTYPE.bind(this, function(type, name, title, toolTip, search, bucket){
				try{
					if( item.type=="Contact" || bucket.name.indexOf(this._gtdinboxPage.settings.labels.types.Projects.prefix)>-1 ){	// It's a project
						// Render a line item
						var elLi = this._doc.createElement("DIV");
							elLi.className = "line-item";
							elLi.innerHTML = title;
							elLi.title = toolTip;
							elLi.setAttribute("gtdSearch", item.search + " " + search);
						item.section.elContent.appendChild(elLi);
						liCounter.count++;
					}
				}catch(e){
					if( GTDDEBUGMODE ) alert("TaskPaneUI _refreshCallbackRender renderThreadListByType\n" + e.toString());
				}
			}), GIPROTOTYPE.bind(this, function(){	
					// Final callback
					this._updateAccordionTitleCount(item, liCounter.count );
				}));
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("TaskPaneUI _refreshCallbackRender\n" + e.toString());
}
};
TaskPaneUI.prototype._updateAccordionTitleCount = function(item, count){
try{
	// Set the required disabled/count settings
	if( count==0 ){	// Nothing found
		item.accordion.disableSection(item.title, true);
	}else{
		item.accordion.disableSection(item.title, false);
	} 
	item.section.elTitleSpan.innerHTML = item.section.elTitleSpan.innerHTML.replace(/\s*\(\d+\)\s*$/,"") + " (" + count + ")";
}catch(e){
	if( GTDDEBUGMODE ) alert("TaskPaneUI _updateAccordionTitleCount\n" + e.toString());
}
};


TaskPaneUI.prototype._click = function(event){
	var el = GIPROTOTYPE.element(event);
	if( el.getAttribute("gtdSearch") ){
		this._gtdinboxPage._searchBoxUI.search( el.getAttribute("gtdSearch") );
	}
};