/* Copyright Andy Mitchell 2007. No warranties */

function ThreadListPopupUI(gtdinboxPage, elParent, inTL){
	this.gtdinboxPage = gtdinboxPage;
	this.doc = gtdinboxPage.doc;
	this.settings = gtdinboxPage.settings;
	this._labels = gtdinboxPage._labels;
	this.threadList = gtdinboxPage._threadList;
	this.threadListUI = gtdinboxPage._threadListUI;
		
	this.eventObservers = new GtdEventObservers(this);
		
	this.create(elParent);
	this.popuper = new GtdPopuper( this.gtdinboxPage, this.el, this );
	this.popuper.addEventHandler( this, "popup", this.onPopup );
	this.popuper.addEventHandler( this, "popupOff", this.onPopupOff );
	this.popuper.addDeadZoneFunction( GIPROTOTYPE.bind(this, this.testPopupDeadZone) );
	
	//this._labels.addEventHandler( this, "refresh-labels", this.labelsUpdate );
	
	this.inTL = inTL;
};



ThreadListPopupUI.prototype.create = function(elParent){

try{
	this.el = this.doc.createElement("DIV");
		this.el.className = "thread-list-popup";
		this.el.style.zIndex = 5;
	if( elParent ) elParent.appendChild( this.el );
			
	var elHeader = this.doc.createElement("DIV");
		elHeader.className = "header";
	this.el.appendChild( elHeader );
	this.elTitle = this.doc.createElement("SPAN");
		this.elTitle.className = "title";
		this.elTitle.innerHTML = this.gtdinboxPage.stringBundle.getString("ThreadListPopupUI.TitleLoading");
	elHeader.appendChild( this.elTitle );
	var elClose = this.doc.createElement("IMG");
		elClose.src = "chrome://gtdinbox/content/cross.png";
		elClose.className = "close-button";
		this.eventObservers.observe(elClose, "mousedown", this.clickClose, false);
	elHeader.appendChild(elClose);
	
	this.elLoading = this.doc.createElement("DIV");
		this.elLoading.className = "loading";
		this.elLoading.style.display = "none";
	this.el.appendChild( this.elLoading );
	this.elLoading.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif'/>";
	
	this.elContent = this.doc.createElement("DIV");
	this.el.appendChild( this.elContent );
	
	this.threadActions = new ThreadActionsUI(this.gtdinboxPage);
	elHeader.appendChild( this.threadActions.el );
	this.threadActions.addEventHandler(this, "click-finished", this.clickClose);
	this.labelSelect = new LabelSelectUI(this.gtdinboxPage, this.elContent, null, this.threadActions);
	this.labelSelect.addEventHandler(this, "click-label", this.toggleLabel);
		
	this.threadPreview = new ThreadPreviewUI(this.gtdinboxPage);
	this.previewShowhider = new Showhider( this.gtdinboxPage, "threadlistpopup-preview", {doc: this.doc, elParent: this.elContent, elContent: this.threadPreview.el, title: this.gtdinboxPage.stringBundle.getString("ThreadListPopupUI.PreviewMessage"), show: true});
	GIPROTOTYPE.addClassName( this.previewShowhider.el, "previewer" );

}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadListPopupUI create\n" + e.toString());
}
};



/* LOADING */

ThreadListPopupUI.prototype.loading = function(v){
try{
	if( typeof v=="undefined" ) return this.elContent.style.display=="none";
	this.elContent.style.display = v? "none" : "";
	this.elLoading.style.display = v? "" : "none";
}catch(e){
	if( GTDDEBUGMODE ) alert("ThreadListPopupUI loading\n" + e.toString());
}
};

ThreadListPopupUI.prototype.load = function(thread){
	this.loading(true);
	this.thread = thread;	
	this.thread.initEventHandler();
	this.thread.eventHandlers.reset();	// get it exclusively for us
		
	//this.gtdinboxPage._recentItems.addThread(this.thread);
	
	
	var fAfterLabelRefresh = GIPROTOTYPE.bind(this, function(){
		this.labelSelect.refresh(this.thread);
		this.threadActions.refresh(this.thread);
		this.threadPreview.load(this.thread);
		
		this.loading(false);
	});
	
	this.thread.refreshData(fAfterLabelRefresh, "ajax");
	
};

ThreadListPopupUI.prototype.loadByRow = function(row){
	this.loading(true);
	this.elTitle.innerHTML = row.subject;
	
		
	row.thread( GIPROTOTYPE.bind(this, function(thread){	// Get, but do not force a deep load
		
		this.thread = thread;
		if( this.thread ){
			this.load(this.thread);
			
			// Set up a full refresh to occur in the background
			setTimeout( GIPROTOTYPE.bind(this, function(){
				thread.refreshData( GIPROTOTYPE.bind(this, function(thread){
					this.labelSelect.refresh(thread);
				}) );
			}), 5000 );
		}else{
			this.retryRowThread(true, row);
		}
		
		
		
	}) );
};
ThreadListPopupUI.prototype.retryRowThread = function(v, row){
	if( this._retryInterval ){
		clearInterval( this._retryInterval );
		this._retryInterval = true;
	}
	if( v ){
		var fRetry = GIPROTOTYPE.bind(this, function(){
				row.thread(GIPROTOTYPE.bind(this, function(thread){	// Force a reload
					if( thread ){
						this.retryRowThread(false);	// Stop the interval
						this.load(thread);
					}
				}), true);
			});
		this._retryInterval = setInterval( fRetry, 1500 );
	}
};




/* POPUP EVENTS */
ThreadListPopupUI.prototype.onPopup = function(){
	if( this.inTL ) this._checked = this.threadListUI.checkRows(null, false);
	
	
	this.doc.body.firstChild.style.opacity = "0.7";
	
	
};
ThreadListPopupUI.prototype.onPopupOff = function(){
	if( this.inTL && this._checked ){
		this.threadListUI.checkRows( this._checked );
		this._checked = null;
	}
	
	this.doc.body.firstChild.style.opacity = "1";
};
ThreadListPopupUI.prototype.testPopupDeadZone = function(el){
	// Return true if a dead zone, thus stopping the popup from closing
	
	// Ensure it's not the New Label Ok/Cancel that is clicked:
	var elNewLabel = this.gtdinboxPage.gmailDOM.element("new-label-box");
	if( elNewLabel ){
		while( el.parentNode ){
			if( el==elNewLabel ) return true;
			el = el.parentNode;
		}
	}
};

ThreadListPopupUI.prototype.clickClose = function(event){
	this.retryRowThread(false);
	this.popupOff();
};

/* EXTERNAL UPDATE EVENTS */
/*
ThreadListPopupUI.prototype.threadUpdate = function(){
	// TODO Only if visible
	if( !this.poppedUp() ) return;
	this.labelSelect.refresh(this.thread);
	this.threadActions.refresh(this.thread);
	this.threadPreview.load(this.thread);
};
*/

/*
ThreadListPopupUI.prototype.labelsUpdate = function(){
	// TODO Only if visible
try{
	if( !this.poppedUp() ) return;
	//this.threadListUI.uncheckOutOfView();	// causes problems on sent items
	this._labels.getActiveLabels(this, function(activeLabels){
			this.thread.verifyLabels( activeLabels );
			this.labelSelect.refresh(this.thread);
			this.threadActions.refresh(this.thread);
			this.threadPreview.load(this.thread);
		});
}catch(e){
	if( GTDDEBUGMODE ) alert("threadlistpopupui labelsUpdate\n" + e.toString());
}
};
*/

ThreadListPopupUI.prototype.toggleLabel = function(e, label){
	// Inform Gmail of change
		
try{
	
	/*
	// Toggle:
	var labelname = label.labelname;
	var fRefresh = GIPROTOTYPE.bind(this, function(){
		this.labelSelect.refresh();	// Update labels if multiple ones were affected
	});
	if( this.thread.hasLabel( labelname ) ){
		this.thread.removeLabel( labelname, fRefresh, "moreActions" );
	}else{
		var fComplete = GIPROTOTYPE.bind(this, function(){
			if( label.doubleClick || (labelname==(this.gtdinboxPage.settings.labels.types.Statuses.prefix + this.gtdinboxPage.settings.labels.types.Statuses.defaults.Finished)) ){	// clicked finish, archive it
				var threadActionsUI = new ThreadActionsUI(this.gtdinboxPage, null, true);
				threadActionsUI._clickArchive(null, GIPROTOTYPE.bind(this, function(){this.clickClose()}) );
			}
			 fRefresh();
		});
		this.thread.addLabel( labelname, fComplete, "moreActions" );
	}
	*/
	
	/*
	this.gtdinboxPage.unsafeWindow.console.log("\n\n===\ntoggle: " + label.labelname);
	this.gtdinboxPage.unsafeWindow.console.log(label);
	this.gtdinboxPage.unsafeWindow.console.log("pre labels:");
	this.gtdinboxPage.unsafeWindow.console.log(GIPROTOTYPE.clone(this.thread.labels));
	this.gtdinboxPage.unsafeWindow.console.log("---\n\n");
	*/
	
	// Toggle:
	var labelname = label.labelname;
	var fRefresh = GIPROTOTYPE.bind(this, function(){
		/*
		this.gtdinboxPage.unsafeWindow.console.log("\n\n===\ntoggle callback: " + label.labelname);
		this.gtdinboxPage.unsafeWindow.console.log("post labels:");
		this.gtdinboxPage.unsafeWindow.console.log(GIPROTOTYPE.clone(this.thread.labels));
		this.gtdinboxPage.unsafeWindow.console.log("---\n\n");
		*/
		
		setTimeout( GIPROTOTYPE.bind(this, function(){
			this.gtdinboxPage.gmailDOM.refreshView();
		}), 200 );
		setTimeout( GIPROTOTYPE.bind(this, function(){
			this.gtdinboxPage.gmailDOM.refreshView();
		}), 800 );
	});
	/*
	this.gtdinboxPage.unsafeWindow.console.log("pre - checking - labels:");
	this.gtdinboxPage.unsafeWindow.console.log(GIPROTOTYPE.clone(this.thread.labels));
	*/
	if( this.thread.hasLabel( labelname ) ){
		//this.gtdinboxPage.unsafeWindow.console.log("popupui removing label [" + labelname + "]");
		this.thread.removeLabel( labelname, fRefresh, "ajax" );
	}else{
		var fComplete = GIPROTOTYPE.bind(this, function(){
			if( label.doubleClick || (labelname==(this.gtdinboxPage.settings.labels.types.Statuses.prefix + this.gtdinboxPage.settings.labels.types.Statuses.defaults.Finished)) ){	// clicked finish, archive it
				var threadActionsUI = new ThreadActionsUI(this.gtdinboxPage, null, true);
				threadActionsUI._clickArchive(null, GIPROTOTYPE.bind(this, function(){this.clickClose()}) );
			}
			fRefresh();
		});
		this.thread.addLabel( labelname, fComplete, "ajax" );
	}
	
	
}catch(e){
	if( GTDDEBUGMODE ) alert("threadlistpopupui clicklabel\n" + e.toString());
}
};




/*
ThreadListPopupUI.prototype.setRow = function(row){
	// Update header, refresh labels (+ set selected), update preview
	
	try{
		this.elTitle.innerHTML = row.subject;
			
		row.thread( GIPROTOTYPE.bind(this, function(thread){
			this.thread = thread;
			if( this.thread ){
				this._setRow_refreshUI(this.thread, row);
			}else{
				this.retryRowThread(true,row);
			}
		}) );

	}catch(e){
		if( GTDDEBUGMODE ) alert("popupui setRow\n" + e.toString());
	}
};
ThreadListPopupUI.prototype._setRow_refreshUI = function(thread, row){
try{
	if( !thread ) return;
	this.thread = thread;
	
	
	this.thread.initEventHandler();
	this.thread.eventHandlers.reset();	// get it exclusively for us
	
	
	this.gtdinboxPage._recentItems.addThread(this.thread);
	
	//this.threadListUI.uncheckOutOfView();	// causes problems on sent items
	this._labels.getActiveLabels(this, function(activeLabels){
			this._setRow_refreshUI_ActiveLabelsCallback(activeLabels, row);
		});
	//this._labels.getActiveLabels(null, GIPROTOTYPE.bind(this, function(activeLabels){this._setRow_refreshUI_ActiveLabelsCallback(activeLabels)}) );
	
}catch(e){
	if( GTDDEBUGMODE ) alert("popupui _setRow_refreshUI\n" + e.toString());
}
};
ThreadListPopupUI.prototype._setRow_refreshUI_ActiveLabelsCallback = function(activeLabels, row){

	var labels = this.thread.verifyLabels( activeLabels );	// ensure it's up to date
			
	this.labelSelect.refresh(this.thread);
	//this.labelSelect.newLabels( this.thread.getUnknownBodyLabels() );
	if( row ){
		while( row.executeAfterNextPopupRefresh.length > 0 ){
			row.executeAfterNextPopupRefresh[0]();
			row.executeAfterNextPopupRefresh.splice(0,1);
		}
	}
	this.threadActions.refresh(this.thread);
		
	this.elPreview.innerHTML = this.thread.getLatestMsg();
	
};
*/






