function ThreadPreviewUI(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._eventObservers = new GtdEventObservers(this);
	
	this.create();
}

ThreadPreviewUI.prototype.create = function(){
try{
	this.el = this._doc.createElement("DIV");
		this.el.className = "";
	
	// Navigation
	this.elNav = this._doc.createElement("DIV");
		this.elNav.className = "navigation";
	
	this.elNavPrevious = this._doc.createElement("DIV");
		this.elNavPrevious.className = "navigation-previous";
		this.elNavPrevious.innerHTML = this._gtdinboxPage.stringBundle.getString("ThreadPreview.Previous");
	this.elNav.appendChild( this.elNavPrevious );
	
	this.elNavIndex = this._doc.createElement("DIV");
		this.elNavIndex.className = "navigation-index";
	this.elNav.appendChild( this.elNavIndex );
	
	this.elNavNext = this._doc.createElement("DIV");
		this.elNavNext.className = "navigation-next";
		this.elNavNext.innerHTML = this._gtdinboxPage.stringBundle.getString("ThreadPreview.Next");
	this.elNav.appendChild( this.elNavNext );
	
	var elClear = this._doc.createElement("DIV");
		elClear.style.clear = "both";
	this.elNav.appendChild( elClear );
	
	this.el.appendChild( this.elNav );
	
	// Preview
	var elPreviewContainer = this._doc.createElement("DIV");
		elPreviewContainer.className = "preview-box";
	this.el.appendChild( elPreviewContainer );
	
	this.elPreview = this._doc.createElement("DIV");
		this.elPreview.className = "preview";
	elPreviewContainer.appendChild( this.elPreview );
	
	// Events
	this._eventObservers.observe( this.elNavPrevious, "click", this._clickPrevious, false );
	this._eventObservers.observe( this.elNavNext, "click", this._clickNext, false );
	this._domWheel = new GtdMouseWheel(this.elNav);
	this._domWheel.addEventHandler( this, "scrollup", this._clickPrevious );	
	this._domWheel.addEventHandler( this, "scrolldown", this._clickNext );	
}catch(e){
	if( GTDDEBUGMODE ) alert( "ThreadPreviewUI create\n" + e.toString() );
}
};

ThreadPreviewUI.prototype.load = function(thread){
	// Set up the indices, load the latest message
	
	this.thread = thread;
	this.index = thread.messages().length - 1;
	this.refreshView();
};

ThreadPreviewUI.prototype._clickPrevious = function(event){
	if( this._navDisabled(this.elNavPrevious) ) return;
	
	this.index--;
	this.refreshView();
};

ThreadPreviewUI.prototype._clickNext = function(event){
	if( this._navDisabled(this.elNavNext) ) return;
	
	this.index++;
	this.refreshView();
};

ThreadPreviewUI.prototype.refreshView = function(){
try{
	var messages = this.thread.messages();
	if( messages.length==0 || (this.index+1)==messages.length ){
		this._navDisabled( this.elNavNext, true );
	}else{
		this._navDisabled( this.elNavNext, false );
	}
	if( this.index>0 ){
		this._navDisabled( this.elNavPrevious, false );
	}else{
		this._navDisabled( this.elNavPrevious, true );
	}
	if( messages.length==0 ){
		this.elNavIndex.innerHTML = "0/0";
		this.elPreview.innerHTML = this._gtdinboxPage.stringBundle.getString("ThreadPreview.NoMessages");	
	}else{
		this.elNavIndex.innerHTML = (this.index + 1) + "/" + messages.length;
		this.elPreview.innerHTML = messages[this.index].msg;	
	}
}catch(e){
	if( GTDDEBUGMODE ) alert( "ThreadPreviewUI refreshView\n" + e.toString() );
}
};

ThreadPreviewUI.prototype._navDisabled = function(elNav, v){
	if( typeof v=="undefined" ) return GIPROTOTYPE.hasClassName(elNav, "disabled");
	if( v ){
		GIPROTOTYPE.addClassName( elNav, "disabled" );
	}else{
		GIPROTOTYPE.removeClassName( elNav, "disabled" );
	}
};