function GtdQueue(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._queue = [];
}

GtdQueue.prototype.createQueueItemManual = function(f, fArgs, callee, test, callback){
	// Queue up f, with arguments fArgs, in context callee, with the test for completion, and an optional callback. 
	var queueItem = {'start':{'callee':callee, 'callback':f, 'args':fArgs}, 'test':test, 'callback':callback};
	this.createQueueItem(queueItem);
	return queueItem;
}
GtdQueue.prototype.createQueueItem = function(qItem){
try{
	// qItem expected to have {start, test, callback} at least	
	qItem.callee = qItem.callee || this;

	
	this._queue.push(qItem);
	this.nextQueueItem();
}catch(e){
	if( GTDDEBUGMODE ) alert("GtdQueue _createQueueItem\n" + e.toString());
}
};
GtdQueue.prototype.clearQueueItem = function(qItem){
try{
	for( var i = this._queue.length - 1; i>=0; i-- ){
		if( this._queue[i]==qItem ){
			if( this._queue[i].callback ) this._queue[i].callback(this._queue[i].act);
			this._queue.splice(i, 1);
		}
	}
	if( this._queueItem==qItem ){
		this._queueItem = null;
		this._queueLastClear = (new Date()).valueOf();
	}
	this.nextQueueItem();
}catch(e){
	if( GTDDEBUGMODE ) alert("GtdQueue _clearQueueItem\n" + e.toString());
}
};
GtdQueue.prototype.nextQueueItem = function(){
try{
	// Clear out aging items, and test active queued items:
	var d = (new Date()).valueOf() - 1500;
	var toClear = [];
	
	var qItem = this._queueItem;
	if( qItem && qItem.test && qItem.test(qItem) ){
		toClear.push(qItem);
	}
	
	for( var i = 0; i < this._queue.length; i++ ){
		var qItem = this._queue[i];
		if( qItem.date < d ){
			toClear.push(qItem);
		}
	}
	for( var i = 0; i < toClear.length; i++ ) this.clearQueueItem(toClear[i]);
	
	// Try the next one:
	this._tryNextQueueItem();
}catch(e){
	if( GTDDEBUGMODE ) alert("GtdQueue _nextQueueItem\n" + e.toString());
}
};
GtdQueue.prototype._tryNextQueueItem = function(){
try{
	if( !this._queueItem && this._queue.length > 0 ){
		this._queueItem = this._queue[0];
		this._queueItem.date = (new Date()).valueOf();
		//this._queueItem.start(this._queueItem);
		if( this._queueItem.start ){
			var qItem = this._queueItem;
			var d = (new Date()).valueOf() - 300;
			if( (this._queueLastClear||0) > d ){	// Something finished recently
				setTimeout( function(){	
					qItem.start.callback.apply( qItem.start.callee, qItem.start.args );
					}, 300);
			}else{
				qItem.start.callback.apply( qItem.start.callee, qItem.start.args );
			}
		}
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("GtdQueue _tryNextQueueItem\n" + e.toString());
}
};
